#include "XGLContext.h"

#include <stdlib.h>
#include <memory.h>
#include <assert.h>

int
CXGLContext::VertexCallback(CXGLVertex *v, void *handle)
{
	delete v;
	return 1;
}

int
CXGLContext::NormalCallback(CXGLNormal *n, void *handle)
{
	delete n;
	return 1;
}

int
CXGLContext::MaterialCallback(CXGLMaterial *m, void *handle)
{
	delete m;
	return 1;
}

int
CXGLContext::LineStyleCallback(CXGLLineStyle *l, void *handle)
{
	delete l;
	return 1;
}

int
CXGLContext::PointStyleCallback(CXGLPointStyle *p, void *handle)
{
	delete p;
	return 1;
}

int
CXGLContext::TextureCoordinateCallback(CXGLTextureCoordinate *tc, void *handle)
{
	delete tc;
	return 1;
}

int 
CXGLContext::ObjectHandleCallback(CXGLObjectHandle *oh, void *handle)
{
	delete oh;
	return 1;
}

int 
CXGLContext::MeshHandleCallback(CXGLMeshHandle *mh, void *handle)
{
	delete mh;
	return 1;
}

int
CXGLContext::TextureHandleCallback(CXGLTextureHandle *th, void *handle)
{
	delete th;
	return 1;
}

int
CXGLContext::TextureBufferHandleCallback(CXGLTextureBufferHandle *tbh, void *handle)
{
	delete tbh;
	return 1;
}

CXGLContext::CXGLContext()
{
}

CXGLContext::~CXGLContext()
{
}

void
CXGLContext::Init()
{
	ObjectHandles.Init();
	MeshHandles.Init();
	TextureHandles.Init();
	TextureBufferHandles.Init();
	Vertices.Init();
	Normals.Init();
	Materials.Init();
	LineStyles.Init();
	PointStyles.Init();
	TextureCoordinates.Init();
	HasLighting =  0;
	HasBackground = 0;

	NumReferences = 1;
}


void 
CXGLContext::AddRef()
{
	NumReferences ++;
}

void
CXGLContext::ReleaseRef()
{
	NumReferences--;
	if (NumReferences != 0)
		return;

	Vertices.Enumerate(VertexCallback, NULL);
	Normals.Enumerate(NormalCallback, NULL);
	Materials.Enumerate(MaterialCallback, NULL);
	LineStyles.Enumerate(LineStyleCallback, NULL);
	PointStyles.Enumerate(PointStyleCallback, NULL);
	TextureCoordinates.Enumerate(TextureCoordinateCallback, NULL);
	//ObjectHandles.Enumerate(ObjectHandleCallback, NULL);
	//MeshHandles.Enumerate(MeshHandleCallback, NULL);
	//TextureHandles.Enumerate(TextureHandleCallback, NULL);
	//TextureBufferHandles.Enumerate(TextureBufferHandleCallback, NULL);

	ObjectHandles.UnInit();
	MeshHandles.UnInit();
	TextureHandles.UnInit();
	TextureBufferHandles.UnInit();

	Vertices.UnInit();
	Normals.UnInit();
	Materials.UnInit();
	LineStyles.UnInit();
	PointStyles.UnInit();
	TextureCoordinates.UnInit();

	if (Parent != NULL)
		Parent->ReleaseRef();

	delete this;
	
}

CXGLContext*
CXGLContext::GetParentContext()
{
	return Parent;
}

CXGLObjectHandle*
CXGLContext::GetObjectHandleForXGLID(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLObjectHandle *ret;
		if (context->ObjectHandles.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

CXGLMeshHandle*
CXGLContext::GetMeshHandleForXGLID(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLMeshHandle *ret;
		if (context->MeshHandles.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

CXGLTextureHandle*
CXGLContext::GetTextureHandleForXGLID(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLTextureHandle *ret;
		if (context->TextureHandles.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

CXGLTextureBufferHandle*
CXGLContext::GetTextureBufferHandleForXGLID(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLTextureBufferHandle *ret;
		if (context->TextureBufferHandles.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

int
CXGLContext::GetLighting()
{
	return HasLighting;

}

int
CXGLContext::GetBackground()
{
	return HasBackground;
}

CXGLMaterial*
CXGLContext::GetMaterial(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLMaterial *ret;
		if (context->Materials.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

CXGLLineStyle*
CXGLContext::GetLineStyle(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLLineStyle *ret;
		if (context->LineStyles.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

CXGLPointStyle*
CXGLContext::GetPointStyle(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLPointStyle *ret;
		if (context->PointStyles.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

CXGLVertex*
CXGLContext::GetVertex(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLVertex *ret;
		if (context->Vertices.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}


CXGLNormal*
CXGLContext::GetNormal(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLNormal *ret;
		if (context->Normals.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}

CXGLTextureCoordinate*
CXGLContext::GetTextureCoordinate(int xglID)
{
	CXGLContext *context = this;
	while (context != NULL)
	{
		CXGLTextureCoordinate *ret;
		if (context->TextureCoordinates.Get(xglID, &ret))
			return ret;

		context = context->Parent;
	}

	return NULL;
}


void
CXGLContext::SetObjectHandle(int xglID, CXGLObjectHandle *objectHandle)
{
	objectHandle->DefinedXGLID = xglID;
	ObjectHandles.Add(objectHandle);	
}

void
CXGLContext::SetMeshHandle(int xglID, CXGLMeshHandle *meshHandle)
{
	meshHandle->DefinedXGLID = xglID;
	MeshHandles.Add(meshHandle);
}

void
CXGLContext::SetTextureHandle(int xglID, CXGLTextureHandle *textureHandle)
{
	textureHandle->DefinedXGLID = xglID;
	TextureHandles.Add(textureHandle);
}

void
CXGLContext::SetTextureBufferHandle(int xglID, CXGLTextureBufferHandle *textureBufferHandle)
{
	textureBufferHandle->DefinedXGLID = xglID;
	TextureBufferHandles.Add(textureBufferHandle);
}

void
CXGLContext::SetMaterial(int xglID, CXGLMaterial *m)
{
	m->DefinedXGLID = xglID;
	Materials.Add(m);
}

void
CXGLContext::SetLineStyle(int xglID, CXGLLineStyle *l)
{
	l->DefinedXGLID = xglID;
	LineStyles.Add(l);
}

void
CXGLContext::SetPointStyle(int xglID, CXGLPointStyle *p)
{
	p->DefinedXGLID = xglID;
	PointStyles.Add(p);
}

void
CXGLContext::SetBackground()
{
	HasBackground = 1;
}

void
CXGLContext::SetLighting()
{
	HasLighting = 1;
}

void
CXGLContext::SetVertex(int xglID, CXGLVertex *v)
{
	v->DefinedXGLID = xglID;
	Vertices.Add(v);
}

void
CXGLContext::SetNormal(int xglID, CXGLNormal *n)
{
	n->DefinedXGLID = xglID;
	Normals.Add(n);
}

void
CXGLContext::SetTextureCoordinate(int xglID, CXGLTextureCoordinate *tc)
{
	tc->DefinedXGLID = xglID;
	TextureCoordinates.Add(tc);
}
