#ifndef __XGLContext_h
#define __XGLContext_h


class CXGLObjectHandle;
class CXGLDataHandle;
class CXGLMeshHandle;
class CXGLTextureHandle;
class CXGLMaterialHandle;
class CXGLLineStyleHandle;
class CXGLPointStyleHandle;
class CXGLVertexHandle;
class CXGLNormalHandle;
class CXGLTextureCoordinateHandle;

#include "XGLTypes.h"
#include "XGLHandleTypes.h"
#include "Utility/HashTable.h"


class CObjHashFunctions : public CHFIntToPointer<CXGLObjectHandle>
{
public:
	static int GetHashKey(CXGLObjectHandle *o){return o->DefinedXGLID;};
};

class CMeshHashFunctions : public CHFIntToPointer<CXGLMeshHandle>
{
public:
	static int GetHashKey(CXGLMeshHandle *m){return m->DefinedXGLID;};
};

class CTextureHashFunctions : public CHFIntToPointer<CXGLTextureHandle>
{
public:
	static int GetHashKey(CXGLTextureHandle *t){return t->DefinedXGLID;};
};

class CTextureBufferHashFunctions : public CHFIntToPointer<CXGLTextureBufferHandle>
{
public:
	static int GetHashKey(CXGLTextureBufferHandle *t){return t->DefinedXGLID;};
};

class CVertexHashFunctions : public CHFIntToPointer<CXGLVertex>
{
public:
	static int GetHashKey(CXGLVertex *v){return v->DefinedXGLID;};
};

class CNormalHashFunctions : public CHFIntToPointer<CXGLNormal>
{
public:
	static int GetHashKey(CXGLNormal *n){return n->DefinedXGLID;};
};

class CMaterialHashFunctions : public CHFIntToPointer<CXGLMaterial>
{
public:
	static int GetHashKey(CXGLMaterial *m){return m->DefinedXGLID;};
};

class CLineStyleHashFunctions : public CHFIntToPointer<CXGLLineStyle>
{
public:
	static int GetHashKey(CXGLLineStyle *l){return l->DefinedXGLID;};
};

class CPointStyleHashFunctions : public CHFIntToPointer<CXGLPointStyle>
{
public:
	static int GetHashKey(CXGLPointStyle *p){return p->DefinedXGLID;};
};

class CTextureCoordinateHashFunctions : public CHFIntToPointer<CXGLTextureCoordinate>
{
public:
	static int GetHashKey(CXGLTextureCoordinate *tc){return tc->DefinedXGLID;};
};


//contexts
class CXGLContext
{
public:

	CXGLContext();
	~CXGLContext();

	static int VertexCallback(CXGLVertex *v, void *handle);
	static int NormalCallback(CXGLNormal *n, void *handle);
	static int MaterialCallback(CXGLMaterial *m, void *handle);
	static int LineStyleCallback(CXGLLineStyle *l, void *handle);
	static int PointStyleCallback(CXGLPointStyle *p, void *handle);
	static int TextureCoordinateCallback(CXGLTextureCoordinate *tc, void *handle);
	static int TextureBufferHandleCallback(CXGLTextureBufferHandle *t, void *handle);
	static int ObjectHandleCallback(CXGLObjectHandle *oh, void *handle);
	static int MeshHandleCallback(CXGLMeshHandle *mh, void *handle);
	static int TextureHandleCallback(CXGLTextureHandle *th, void *handle);

	void Init();

	CXGLContext *Parent;
	CXGLContext *GetParentContext();

	void AddRef();
	void ReleaseRef();
	
	//Get Functions
	CXGLObjectHandle* GetObjectHandleForXGLID(int xglID);
	CXGLMeshHandle* GetMeshHandleForXGLID(int xglID);
	CXGLTextureHandle* GetTextureHandleForXGLID(int xglID);
	CXGLTextureBufferHandle* GetTextureBufferHandleForXGLID(int xglID);
	CXGLMaterial* GetMaterial(int xglID);
	CXGLLineStyle* GetLineStyle(int xglID);
	CXGLPointStyle* GetPointStyle(int xglID);
	CXGLVertex* GetVertex(int xglID);
	CXGLNormal* GetNormal(int xglID);
	CXGLTextureCoordinate* GetTextureCoordinate(int xglID);
	int GetLighting();
	int GetBackground();

	//Set Functions
	void SetObjectHandle(int xglID, CXGLObjectHandle *objectHandle);
	void SetMeshHandle(int xglID, CXGLMeshHandle *meshHandle);
	void SetTextureHandle(int xglID, CXGLTextureHandle *textureHandle);
	void SetTextureBufferHandle(int xglID, CXGLTextureBufferHandle *textureBufferhandle);
	void SetMaterial(int xglID, CXGLMaterial *materialHandle);
	void SetLineStyle(int xglID, CXGLLineStyle *lineStyleHandle);
	void SetPointStyle(int xglID, CXGLPointStyle *pointStyleHandle);
	void SetVertex(int xglID, CXGLVertex *vertexHandle);
	void SetNormal(int xglID, CXGLNormal *normalHandle);
	void SetTextureCoordinate(int xglID, CXGLTextureCoordinate *textureCoordinate);
	void SetLighting();
	void SetBackground();
	


protected:
	CHashTable<int, CXGLObjectHandle*, CObjHashFunctions, false, CHTNew> ObjectHandles;
	CHashTable<int, CXGLMeshHandle*, CMeshHashFunctions, false, CHTNew> MeshHandles;
	CHashTable<int, CXGLTextureHandle*, CTextureHashFunctions, false, CHTNew> TextureHandles;
	CHashTable<int, CXGLTextureBufferHandle*, CTextureBufferHashFunctions, false, CHTNew> TextureBufferHandles;
	CHashTable<int, CXGLVertex*, CVertexHashFunctions, false, CHTNew> Vertices;
	CHashTable<int, CXGLMaterial*, CMaterialHashFunctions, false, CHTNew> Materials;
	CHashTable<int, CXGLLineStyle*, CLineStyleHashFunctions, false, CHTNew> LineStyles;
	CHashTable<int, CXGLPointStyle*, CPointStyleHashFunctions, false, CHTNew> PointStyles;
	CHashTable<int, CXGLNormal*, CNormalHashFunctions, false, CHTNew> Normals;
	CHashTable<int, CXGLTextureCoordinate*, CTextureCoordinateHashFunctions, false, CHTNew> TextureCoordinates;
	int HasLighting;
	int HasBackground;

	int NumReferences;


	//these may be used later
	//A map from XGLID to Postion
	//A map from XGLID to Normal


};

#endif