#ifndef __XGLHandleTypes_h
#define __XGLHandleTypes_h

class CXGLObject;
class CXGLContext;
class CXGLMesh;

class CXGLObjectHandle
{
public:
	/*
	An object can move through three states

  	UnDefined
		CallbackHandle = -1
		XGLIDObjectReferenceID is the ID that the object should resolve to in the context
		Object = NULL;
		ObjectPathID is the pathID of the object
		Context is the context in which the ID should be resolved

  	Incomplete
		CallbackHandle = -1
		XGLIDObjectReferenceID = -1
		Object is an object that has unresolved references
		ObjectPathID is the pathID of the object
		Context = NULL

	Output
		CallbackHandle is value returned by the Callback
		XGLIDObjectReferenceID = -1
		ObjectHandle = NULL
		ObjectPathID is the pathID of the object
		Context = NULL
	*/

	

	int XGLIDObjectReferenceID;
	int ObjectPathID;
	int CallbackHandle;
	CXGLObject *Object;
	CXGLContext *Context;

	//Used by contexts only
	int DefinedXGLID; //The XGLID that this handle is a definition for.
};

class CXGLMeshHandle
{
public:
	//If Mesh is NULL, CallbackHandle contains the handle that should be used when calling 
	//back with this mesh.

	int XGLIDMeshReferenceID;
	int CallbackHandle;
	CXGLMesh *Mesh;
	CXGLContext *Context;

	int DefinedXGLID; //The XGLID that this handle is a definition for.
};

class CXGLTextureHandle
{
public:

	int XGLIDTextureReferenceID;
	int CallbackHandle;
	CXGLTexture *Texture;
	CXGLContext *Context;

	//Used by contexts only
	int DefinedXGLID; //The XGLID that this handle is a definition for.
};

class CXGLTextureBufferHandle
{
public:

	int XGLIDTextureBufferReferenceID;
	int CallbackHandle;
	CXGLContext *Context;

	//Used by contexts only
	int DefinedXGLID; //The XGLID that this handle is a definition for.
};

#endif