#ifndef __XGLImport_h
#define __XGLImport_h


#include "XGLTypes.h"
#include "XGLResolver.h"
#include "XGLContext.h"
#include "xmlparse.h"
#include "Utility/VVHashTable.h"
#include <stdio.h>


/****************************************

The following classes are used internally
and may be changed by RealityWave at any
time.  You should not use them.  The 
publicly usable classes appear later in this file

****************************************/

class CXGLStatusCallback;

class CElementRequires
{
	CResizeableArray<int> TagArray;
	
public:
	int HowMany(int tag);

	void AddElement(int tagNum);
	void Init();
};

class CStateStackElem
{
public:
	int Data;
	CStateStackElem* Next;

};

class CStateStack
{

public:
	CStateStackElem* Top;
	void Push(int data);
	int Pop();
	int GetTopValue();
	CStateStack();
	~CStateStack();
};

class CPathIDHashFunctions : public CVVHTNewDelete
{
public:
	static unsigned int GetHash1ForValue(const int* value){return *value;};
	static unsigned int GetHash2ForValue(const int* value){return *value;}; 
	static bool GetDoKeysMatch(const int* v1, const int* v2){return (*v1 == *v2);};
};

typedef CVVHashTable<int, CPathIDHashFunctions> CPathIDHash;

class CObjectStackElem
{
public:

	//data
	CXGLObject *Object;
	CPathIDHash PathIDHash;
	CElementRequires ObjectRequires;

	//next element
	CObjectStackElem* Next;
};

class CObjectStack
{
public:
	
	CObjectStackElem *Top;
	void Push(CXGLObject *object);
	CObjectStackElem *GetTop();
	CXGLObject* Pop();
	CObjectStack();
	~CObjectStack();
};

	enum Tag
	{
		WorldTag = 0,
		ObjectTag = 1,
		LightingTag = 2,
		TransformTag = 3,
		MeshTag = 4,
		TransformForwardTag = 5,
		TransformUpTag = 6,
		TransformPositionTag = 7,
		TransformScaleTag = 8,
		MaterialTag = 9,
		FaceTag = 10,
		FaceVertexTag1 = 11,
		VertexPositionTag = 12,
		NormalTag = 13,
		MaterialRefTag = 14,
		IDTag = 15,
		MaterialEmissiveTag = 16,
		MaterialDiffuseTag = 17,
		MaterialSpecularTag = 18,
		MaterialAmbientTag = 19,
		MaterialShininessTag = 20,
		MaterialAlphaTag = 21,
		SurfaceTag = 22,
		OtherTag = 23,
		LightDiffuseTag = 24,
		LightSpecularTag = 25,
		LightDirectionTag = 26,
		LightAmbientTag = 27,
		DirectionalLightTag = 28,
		BackgroundTag = 29,
		BackgroundColorTag = 30,
		VertexRefTag = 31,
		NormalRefTag = 32,
		MeshRefTag = 33,
		DataTag = 34,
		StringTag = 35,
		BinTag = 36,
		FaceVertexTag2 = 37,
		FaceVertexTag3 = 38,
		PatchTag = 39,
		TextureCoordinateTag = 40,
		TextureCoordinateRefTag = 41,
		TextureTag = 42,
		TextureRefTag = 43,
		TextureRGBTag = 44,
		TextureRGBRefTag = 45,
		TextureRGBATag = 46,
		TextureRGBARefTag = 47,
		ReplaceTag = 48,
		ModulateTag = 49,
		DecalTag = 50,
		RepeatTag = 51,
		ClampTag = 52,
		WidthTag = 53,
		HeightTag = 54,
		TextureBorderColorTag = 55,
		SmoothTag = 56,
		ObjectRefTag = 57,
		ExternalObjectRefTag = 58,
		RefTag = 59,
		ExtentsTag = 60,
		RefTypeTag = 61,
		SmoothingGroupTag = 62,
		LineTag = 63,
		LineVertexTag1 = 64,
		LineVertexTag2 = 65,
		LineStyleTag = 66,
		LineStyleRefTag = 67,
		LineStyleWidthTag = 68,
		LineStylePatternTag = 69,
		LineStylePatternFactorTag = 70,
		PointTag = 71,
		PointVertexTag1 = 72,
		PointStyleTag = 73,
		PointStyleRefTag = 74, 
		PointStyleSizeTag = 75,
		ExtGeomTag = 76,
		ExtGeomElemTag = 77,
		ExtGeomElemDataTag = 78,
		ExtGeomElemExtentsTag = 79,
		ExtGeomElemPriorityTag = 80,
		ExternalObjectStaticRefTag = 81,
		NameTag = 82,
	};

	enum State
	{
		SpaceState = -1,
		WorldState = 0,
		ObjectState = 2,
		TransformState = 3,
		MeshState = 4,
		MaterialState = 5,
		LightingState = 6,
		BackgroundState = 7,
		DataState = 8,
		FaceState = 9,
		TextureState = 10,
		TextureBufferState = 11,
		FaceVertexState = 12,
		ExternalObjectReferenceState = 13,
		LineState = 14,
		PointState = 15,
		LineStyleState = 16,
		PointStyleState = 17,
		ExtgeomState = 18,
		ExternalObjectStaticReferenceState = 19
	};

	enum ParserState
	{
		CharDataState = 0,
		OtherState = 1
	};

//expat parser callback functions
extern "C" void StartElement(void *userData, const char *name, const char **atts);
extern "C" void EndElement(void *userData, const char *name);
extern "C" void CharElement(void *userData, const char *s, int len);

void CleanStringFromSpacesAndNL(char *s);

class CXGLImport
{
public:
	CXGLImport();
	~CXGLImport();

	//the public functions.....
	void Init();
	void UnInit();

	enum FileType
	{
		ZGLFileType = 0,
		XGLFileType = 1,
		OtherFileType = 2
	};

	//this will import from a file name
	int Import(const char *filename, int fileType, CXGLStatusCallback* progress);
	void StopImporting();
	int IsStopped();


	//this indicates that an outside source will call our start, char, and end
	//functions with data
	CXGLImport* Import(CXGLStatusCallback* progress);

	//set the callback
	void InitCallback(CXGLImportCallback* callback);

	void ImportStartElement(const char *name, const char **atts);
	void ImportEndElement(const char *name);
	void ImportCharElement(const char *s, int len);
	CXGLMesh* GetCurrentMesh() { return CurrentMesh; }

protected:

	void AccumCharElement();
	int GetTag(const char* tagName);

	// parser state
	int CurrentParserState;
	int CurrentState;

	// xml parser buffer vars
	CResizeableArray<char> CopyString;
//	char *CopyString;
//	int CopyStringLength;
	char Buf[BUFSIZ];
	char ErrorString[1000];

	//Space state
	void SpaceStateBeginTagHandler(int tag, const char**atts);
	void SpaceStateCharDataHandler(const char *s);
	void SpaceStateEndTagHandler(int tag);

	//World state
	void WorldStateBeginTagHandler(int tag, const char**atts);
	void WorldStateCharDataHandler(const char *s);
	void WorldStateEndTagHandler(int tag);

	//Object state
	void ObjectStateBeginTagHandler(int tag, const char** atts);
	void ObjectStateCharDataHandler(const char *s);
	void ObjectStateEndTagHandler(int tag);

	//external object reference state
	void ExternalObjectStateBeginTagHandler(int tag, const char** atts);
	void ExternalObjectStateCharDataHandler(const char *s);
	void ExternalObjectStateEndTagHandler(int tag);

	//Lighting state
	void LightingStateBeginTagHandler(int tag, const char**atts);
	void LightingStateCharDataHandler(const char *s);
	void LightingStateEndTagHandler(int tag);

	//Background state
	void BackgroundStateBeginTagHandler(int tag, const char**atts);
	void BackgroundStateCharDataHandler(const char *s);
	void BackgroundStateEndTagHandler(int tag);

	//Material state
	void MaterialStateBeginTagHandler(int tag, const char** atts);
	void MaterialStateCharDataHandler(const char *s);
	void MaterialStateEndTagHandler(int tag);

	//LineStyle state
	void LineStyleStateBeginTagHandler(int tag, const char** atts);
	void LineStyleStateCharDataHandler(const char *s);
	void LineStyleStateEndTagHandler(int tag);

	//PointStyle state
	void PointStyleStateBeginTagHandler(int tag, const char** atts);
	void PointStyleStateCharDataHandler(const char *s);
	void PointStyleStateEndTagHandler(int tag);

	//Transform state
	void TransformStateBeginTagHandler(int tag, const char** atts);
	void TransformStateCharDataHandler(const char *s);
	void TransformStateEndTagHandler(int tag);
	void CheckAndFixTransform();

	//Mesh state
	void MeshStateBeginTagHandler(int tag, const char** atts);
	void MeshStateCharDataHandler(const char *s);
	void MeshStateEndTagHandler(int tag);

	//Data state
	void DataStateBeginTagHandler(int tag, const char** atts);
	void DataStateCharDataHandler(const char *s);
	void DataStateEndTagHandler(int tag);

	//Face state
	void FaceStateBeginTagHandler(int tag, const char** atts);
	void FaceStateCharDataHandler(const char *s);
	void FaceStateEndTagHandler(int tag);		

	//Line state
	void LineStateBeginTagHandler(int tag, const char** atts);
	void LineStateCharDataHandler(const char *s);
	void LineStateEndTagHandler(int tag);

	//Point state
	void PointStateBeginTagHandler(int tag, const char** atts);
	void PointStateCharDataHandler(const char *s);
	void PointStateEndTagHandler(int tag);

	//Face vertex state
	void FaceVertexStateBeginTagHandler(int tag, const char** atts);
	void FaceVertexStateCharDataHandler(const char *s);
	void FaceVertexStateEndTagHandler(int tag);

	//Texture state
	void TextureStateBeginTagHandler(int tag, const char** atts);
	void TextureStateCharDataHandler(const char *s);
	void TextureStateEndTagHandler(int tag);		

	//Extended Geometry state
	void ExtgeomStateBeginTagHandler(int tag, const char** atts);
	void ExtgeomStateCharDataHandler(const char *s);
	void ExtgeomStateEndTagHandler(int tag);		

	//Other state
	void OtherStateBeginTagHandler(int tag, const char** atts);
	void OtherStateCharDataHandler(const char *s);
	void OtherStateEndTagHandler(int tag);

	int OtherStateLevel;

	void Error(const char* errorString);

	enum WorldInternalState
	{
		WorldTextureCoordinateState,
		WorldTextureRGBState,
		WorldTextureRGBAState,
		WorldObjectRefState,
		WorldNameState,
		WorldMeshRefState,
		OtherWorldState
	};

	enum TransformInternalState
	{
		ForwardState,
		UpState,
		PositionState,
		ScaleState,
		OtherTransformState
	};

	enum MaterialInternalState
	{
		EmissiveState,
		DiffuseState,
		SpecularState,
		AmbientState,
		ShininessState,
		AlphaState,
		OtherMaterialState
	};

	enum LineStyleInternalState
	{
		LineStyleWidthState,
		LineStylePatternState,
		LineStylePatternFactorState,
		OtherLineStyleState
	};

	enum PointStyleInternalState
	{
		PointStyleSizeState,
		OtherPointStyleState
	};

	enum DataInternalState
	{
		StringState,
		BinState,
		OtherDataState
	};

	enum TextureInternalState
	{
		TextureBorderColorState,
		TextureRGBRefState,
		TextureRGBARefState,
		TextureRGBState,
		TextureRGBAState,
		OtherTextureState
	};

	enum FaceVertexInternalState
	{
		FVPositionState,
		FVNormalState,
		FVNormalRefState,
		FVVertexRefState,
		FVTextureCoordinateRefState,
		FVTextureCoordinateState,
		OtherFVState
	};

	enum MeshInternalState
	{
		MeshPositionState,
		MeshNormalState,
		MeshTextureCoordinateState,
		MeshTextureRGBState,
		MeshTextureRGBAState,
		MeshExtGeomState,
		OtherMeshState
	};

	enum FaceInternalState
	{
		FaceTextureRGBState,
		FaceTextureRGBAState,
		FaceMaterialRefState,
		FaceTextureRefState,
		FaceSmoothState,
		OtherFaceState
	};
	
	enum LineInternalState
	{
		LineTextureRGBState,
		LineTextureRGBAState,
		LineMaterialRefState,
		LineTextureRefState,
		LineStyleRefState,
		OtherLineState
	};

	enum PointInternalState
	{
		PointTextureRGBState,
		PointTextureRGBAState,
		PointMaterialRefState,
		PointTextureRefState,
		PointStyleRefState,
		OtherPointState
	};

	enum LightingInternalState
	{
		LightingAmbientState,
		LightingDirectionState,
		LightingDiffuseState,
		LightingSpecularState,
		OtherLightingState
	};

	enum ObjectInternalState
	{
		ObjectMeshRefState,
		ObjectTextureCoordinateState,
		ObjectTextureRGBState,
		ObjectTextureRGBAState,
		ObjectObjectRefState,
		ObjectNameState,
		OtherObjectState
	};

	enum BackgroundInternalState
	{
		BackgroundColorState,
		OtherBackgroundState
	};


	enum ExternalObjectReferenceInternalState
	{
		ExternalReferenceState,
		ExternalExtentsState,
		ExternalRefTypeState,
		ExternalNameState,
		OtherExternalState
	};

	enum ExtgeomInternalState
	{
		ExtgeomelemState,
		ExtgeomelemdataState,
		ExtgeomelemextentsState,
		ExtgeomelempriorityState,
		OtherExtgeomState,
	};

	int SkipWhiteSpaceAndCommas(char* start, char** next);
	
	//utility routines to retrieve values from xml strings
	int Get2FloatVector(const char* string, float* val1, float *val2);
	int Get3FloatVector(const char* string, float* val1, float *val2, float *val3);
	int Get4FloatVector(const char* string, float* val1, float *val2, float *val3, float *val4);
	int Get6FloatVector(const char* string, float* val1, float *val2, float *val3, float *val4, float *val5, float *val6);
	int Get2DoubleVector(const char* string, double* val1, double* val2);
	int Get3DoubleVector(const char* string, double* val1, double* val2, double* val3);
	int Get4DoubleVector(const char* string, double* val1, double* val2, double* val3, double* val4);
	int Get6DoubleVector(const char* string, double* val1, double* val2, double* val3, double* val4, double *val5, double* val6);
	int GetFloat(const char* string, float* num);
	int GetDouble(const char*, double *num);
	int GetInt(const char* string, int* num);
	int GetUnsignedInt(const char* string, unsigned int* num);
	int GetNormalized3FloatVector(float x, float y, float z, float *newX, float *newY, float* newZ);
	
	int GetBinaryDataFromHex(int hexLen, const char* hexString, unsigned char* dataString);//allocate dataString
	int IsValidHexChar(char c);
	int SetBit(unsigned int bitNum, unsigned int* num);

	//routines to get common attribute values from a list of attributes
	int GetDefinitionID(const char** atts);
	int GetPathID(const char** atts);
	int GetPatchID(const char** atts);
	void GetDefinitionAndPathID(int *defID, int *pathID, const char** atts);
	void GetWidthHeightAndDefinitionID(int *width, int *height, int *defID, const char** atts);

	//data initialization functions
	void InitializeCurrentWorld();
	void InitializeCurrentObject(int pathID, int xglID);
	void InitializeCurrentObjectExternal(int pathID, int IsStatic);		
	void InitializeCurrentData(const char *organization);
	void InitializeCurrentMesh();
	void InitializeCurrentMaterial();
	void InitializeCurrentLineStyle();
	void InitializeCurrentPointStyle();
	void InitializeCurrentLight();
	void InitializeCurrentLighting();
	void InitializeCurrentBackground();
	void InitializeCurrentPatch(int patchID);
	void InitializeCurrentTexture();
	void InitializeCurrentTextureBuffer();
	void InitializeCurrentExtgeom(const char *organization);

	//data uninitialization functions
	void UnInitializeCurrentMesh();
	void UnInitializeCurrentData();

	//resolve verification functions
	int VerifyCurrentWorldResolved();
	int VerifyObjectResolved(CXGLObjectHandle *handle);

	//current state data
	CXGLWorld *CurrentWorld;
	CXGLLight *CurrentLight;
	CXGLLighting *CurrentLighting;
	CXGLBackground *CurrentBackground;
	CXGLData *CurrentData;
	CXGLMesh *CurrentMesh;
	CXGLMaterial CurrentMaterial;
	CXGLLineStyle CurrentLineStyle;
	CXGLPointStyle CurrentPointStyle;
	CXGLVertex *CurrentVertex;
	CXGLTexture *CurrentTexture;
	CXGLTextureBuffer *CurrentTextureBuffer;
	CXGLExtgeom *CurrentExtgeom;
	CXGLExtgeomelem *CurrentExtgeomelem;

	//resizeable arrays for mesh data
	CResizeableArray<CXGLFace> Faces;
	CResizeableArray<CXGLLine> Lines;
	CResizeableArray<CXGLPoint> Points;
	CResizeableArray<CXGLMaterial> Materials;
	CResizeableArray<CXGLLineStyle> LineStyles;
	CResizeableArray<CXGLPointStyle> PointStyles;
	CResizeableArray<CXGLVertex> Vertices;
	CResizeableArray<CXGLNormal> Normals;
	CResizeableArray<CXGLTextureCoordinate> TextureCoordinates;
	CResizeableArray<CXGLPatch*> Patches;
	//CResizeableArray<CXGLTextureHandle*> TextureHandles;
	void CreateCurrentMeshFromResizeableArrays();

	//resizeable arrays for data
	CResizeableArray<char*> StringNames;
	CResizeableArray<unsigned int> StringNameLengths;
	CResizeableArray<char*> Strings;
	CResizeableArray<unsigned int> StringLengths;
	CResizeableArray<char*> BinNames;
	CResizeableArray<unsigned int> BinNameLengths;
	CResizeableArray<unsigned char*> Bins;
	CResizeableArray<unsigned int> BinLengths;
	void CreateCurrentDataFromResizeableArrays();

	//current state definition ids
	int CurrentObjectID;
	int CurrentMeshID;
	int CurrentMaterialID;
	int CurrentLineStyleID;
	int CurrentPointStyleID;
	int CurrentVertexID;
	int CurrentNormalID;
	int CurrentTextureCoordinateID;
	int CurrentTextureBufferID;
	int CurrentTextureID;

	//indicates the current face vertex we're parsing
	int CurrentFaceVertex; //0, 1, or 2
	//indicates the current patch we're parsing
	int CurrentPatchIndex;
	int CurrentPatchHigh;

	//previous reference ids
	unsigned int PrevTextureReferenceID;
	unsigned int PrevMatReferenceID;
	unsigned int PrevLineStyleReferenceID;
	unsigned int PrevPointStyleReferenceID;
	unsigned int PrevVertexReferenceID;
	unsigned int PrevNormalReferenceID;
	unsigned int PrevTextureCoordinateReferenceID;

	//transform manipulation functions
	void SetTransformToIdentity(CXGLTransform *transform);
	void InitializeExtents(CXGLExtents *extents);

	//stacks
	CObjectStack ObjectStack;
	CStateStack StateStack;

	//hash for object pathids in the world
	CPathIDHash TopObjectPathIDHash;

	//contexts
	CXGLContext *CurrentContext;

	// internal states

	int CurrentMaterialState;
	int CurrentLineStyleState;
	int CurrentPointStyleState;
	int CurrentTransformState;
	int CurrentDataState;
	int CurrentTextureState;
	int CurrentLightingState;
	int CurrentBackgroundState;
	int CurrentFaceState;
	int CurrentLineState;
	int CurrentPointState;
	int CurrentFVState;
	int CurrentMeshState;
	int CurrentObjectState;
	int CurrentWorldState;
	int CurrentExternalObjectState;
	int CurrentExtgeomState;
	
	CXGLStatusCallback* Status;

	void UpdateStatus();

	int State;

	int NumBytesInFile;
	int CurrentByte;
	int FileType;

	XML_Parser Parser;

	//the resolver
	CXGLResolver Resolver;

	//flags to determine if we have received the required tags

	/*these can be used anywhere that can't redefine something of the same type
	Basically everything except objects */

	CElementRequires WorldRequires;
	CElementRequires TransformRequires;
	CElementRequires FaceRequires;
	CElementRequires LineRequires;
	CElementRequires PointRequires;
	CElementRequires TextureRequires;
	CElementRequires MaterialRequires;
	CElementRequires LineStyleRequires;
	CElementRequires PointStyleRequires;
	CElementRequires BackgroundRequires;
	CElementRequires LightingRequires;
	CElementRequires DirectionalLightRequires;
	CElementRequires DataRequires;
	CElementRequires FaceVertexRequires;
//	CElementRequires ExtgeomRequires;

	int NeedsData;

};

#endif
