#ifndef __XGLResolver_h
#define __XGLResolver_h

#include "Utility/ResizeableArray.h"

class CXGLContext;
class CXGLImportCallback;

class CXGLWorld;
class CXGLLighting;
class CXGLBackground;
class CXGLObject;
class CXGLData;
class CXGLMesh;
class CXGLMaterial;
class CXGLLineStyle;
class CXGLPointStyle;
class CXGLVertex;
class CXGLNormal;
class CXGLTextureBuffer;
class CXGLTextureCoordinate;
class CXGLTexture;
class CXGLExtgeom;

class CXGLObjectHandle;
class CXGLDataHandle;
class CXGLMeshHandle;
class CXGLTextureHandle;
class CXGLTextureBufferHandle;
class CXGLExtgeomHandle;


class CXGLResolver
{
public:
	//Initialization
	void Init(CXGLImportCallback *callback);
	void UnInit();

	//Contexts
	CXGLContext *CreateContext(CXGLContext *parent);
	CXGLContext *ReleaseContext(CXGLContext *context);

	


	/*
	Definitions

	When an object is added as a definition, ownership of the object and all contained arrays
	passed to the resolver.  The caller should forget about the object and its arrays.

	The context passed in is both the context that the object should be resolved in
	and the context that the resolved object should be added to.  This means that object
	can use references that are siblings.
	*/
	
	void AddObjectDefinition(CXGLContext *addContext, CXGLContext *resolveContext, int xglID, CXGLObject *object);
	void AddTextureDefinition(CXGLContext *context, int xglID, CXGLTexture *texture);
	void AddMeshDefinition(CXGLContext *addContext, CXGLContext *resolveContext, int xglID, CXGLMesh *mesh);
	void AddMaterialDefinition(CXGLContext *context, int xglID, CXGLMaterial *material);
	void AddLineStyleDefinition(CXGLContext *context, int xglID, CXGLLineStyle *lineStyle);
	void AddPointStyleDefinition(CXGLContext *context, int xglID, CXGLPointStyle *pointStyle);
	void AddVertexDefinition(CXGLContext *context, int xglID, CXGLVertex *vertex);
	void AddNormalDefinition(CXGLContext *context, int xglID, CXGLNormal *normal);
	void AddTextureCoordinateDefinition(CXGLContext *context, int xglID, CXGLTextureCoordinate *textureCoordinate);
	void AddTextureBufferDefinition(CXGLContext *context, int xglID, CXGLTextureBuffer *textureBuffer);

	/*
	References

	
	These handles become invalid when the context is released, or when they are passed in to another function.
	*/
	CXGLObjectHandle *CreateObjectReference(CXGLContext *context, int xglID);
	CXGLMeshHandle *CreateMeshReference(CXGLContext *context, int xglID);
	CXGLTextureHandle *CreateTextureReference(CXGLContext *context, int xglID);
	CXGLTextureBufferHandle *CreateTextureBufferReference(CXGLContext *context, int xglID);
	
	/*
	Objects

	When an object is added, ownership of the object and all contained arrays is passed to the resolver.

	The handles returned by these functions are valid until they are passed in as parameter for the first time.
	*/
	void AddWorld(CXGLContext* context, CXGLWorld *world);
	void AddBackground(CXGLContext *context, CXGLBackground *background);
	void AddLighting(CXGLContext *context, CXGLLighting *lighting);
	CXGLDataHandle *AddData(CXGLContext *context, CXGLData *data);
	CXGLObjectHandle *AddObject(CXGLContext *context, CXGLObject *object);
	CXGLMeshHandle *AddMesh(CXGLContext *context, CXGLMesh *mesh);
	CXGLTextureHandle *AddTexture(CXGLContext *context, CXGLTexture *texture);
	CXGLTextureBufferHandle *AddTextureBuffer(CXGLContext *context, CXGLTextureBuffer *textureBuffer);
	CXGLExtgeomHandle *AddExtgeom(CXGLContext *context, CXGLExtgeom *extgeom);

	//error callback passthrough
	void AddError(const char *errorString);
	
protected:

	int CanObjectResolve(CXGLContext *context, CXGLObject *object);
	int CanMeshResolve(CXGLContext *context, CXGLMesh *mesh);
	int CanTextureResolve(CXGLContext *context, CXGLTexture *texture);	
	void ResolveWorld(CXGLContext* context, CXGLWorld* world);

	void CheckAndResolveAllObjects();
	
	CResizeableArray<CXGLObjectHandle*> ObjectHandles;
	CResizeableArray<CXGLMeshHandle*> MeshHandles;
	CResizeableArray<CXGLTextureHandle*> TextureHandles;
	CResizeableArray<CXGLTextureBufferHandle*> TextureBufferHandles;
	

	CXGLImportCallback *Callback;
};

#endif
