#include "XGLTypes.h"
#include "XGLHandleTypes.h"
#include <stdlib.h>

int
CXGLObject::GetMeshCallbackHandle()
{
	if (MeshHandle == 0)
		return -1;
	return MeshHandle->CallbackHandle;
}

int
CXGLWorld::GetMeshCallbackHandle()
{
	if(MeshHandle == 0)
		return -1;

	return MeshHandle->CallbackHandle;
}

int 
CXGLObject::GetObjectCallbackHandle(int index)
{
	CXGLObjectHandle *handle = ObjectHandles.GetElement(index);
	return handle->CallbackHandle;
}

int 
CXGLWorld::GetObjectCallbackHandle(int index)
{
	CXGLObjectHandle *handle = ObjectHandles.GetElement(index);
	return handle->CallbackHandle;
}

int 
CXGLObject::GetDataCallbackHandle(int index)
{
	CXGLDataHandle *handle = DataHandles.GetElement(index);
	return (int)(handle);	
}

int 
CXGLWorld::GetDataCallbackHandle(int index)
{
	CXGLDataHandle *handle = DataHandles.GetElement(index);
	return (int)(handle);	
}

int
CXGLMesh::GetTextureCallbackHandle(int index)
{
	CXGLTextureHandle *handle = TextureHandles.GetElement(index);
	return handle->CallbackHandle;
}

int 
CXGLPatch::GetDataCallbackHandle(int index)
{
	CXGLDataHandle *handle = DataHandles.GetElement(index);
	return (int)(handle);
}

int
CXGLTexture::GetTextureBufferCallbackHandle()
{
	if (TextureBufferHandle == 0)
		return -1;
	return TextureBufferHandle->CallbackHandle;
}

//constructors & deconstructors
CXGLWorld::CXGLWorld()
{
	NumObjects = 0;
	MeshHandle = NULL;
	Name = NULL;
}

CXGLWorld::~CXGLWorld()
{
	NumObjects = 0;
	
	if (Name != NULL)
	{
		delete[] Name;
		Name = NULL;
	}
}

CXGLObject::CXGLObject()
{
	IsInResolveLoop = 0;
	NumObjects = 0;
	NumDatas = 0;
	NumExtgeoms = 0;
	Ref = NULL;
	RefType = NULL;
	MeshHandle = NULL;
	Name = NULL;
}

CXGLObject::~CXGLObject()
{
	NumObjects = 0;
	NumDatas = 0;

	if (Ref != NULL)
	{
		delete[] Ref;
		Ref = NULL;
	}

	if (RefType != NULL)
	{
		delete[] RefType;
		RefType = NULL;
	}

	if (Name != NULL)
	{
		delete[] Name;
		Name = NULL;
	}
}

CXGLMesh::CXGLMesh()
{
	DoubleSided = 0;

	NumFaces = 0;
	Faces = NULL;

	NumLines = 0;
	Lines = NULL;

	NumPoints = 0;
	Points = NULL;

	NumMaterials = 0;
	Materials = NULL;

	NumLineStyles = 0;
	LineStyles = NULL;

	NumPointStyles = 0;
	PointStyles = NULL;

	NumNormals = 0;
	Normals = NULL;

	NumTextureCoordinates = 0;
	TextureCoordinates = NULL;

	NumTextures = 0;

	NumVertices = 0;
	Vertices = NULL;

	NumPatches = 0;
	Patches = NULL;

	NumExtgeoms = 0;
}

CXGLMesh::~CXGLMesh()
{
	NumFaces = 0;
	if (Faces != NULL)
		delete[] Faces;
	Faces = NULL;
	
	NumLines = 0;
	if (Lines != NULL)
		delete[] Lines;
	Lines = NULL;

	NumPoints = 0;
	if (Points != NULL)
		delete[] Points;
	Points = NULL;

	NumMaterials = 0;
	if (Materials != NULL)
		delete[] Materials;
	Materials = NULL;
	
	NumLineStyles = 0;
	if (LineStyles != NULL)
		delete[] LineStyles;
	LineStyles = NULL;

	NumPointStyles = 0;
	if (PointStyles != NULL)
		delete[] PointStyles;
	PointStyles = NULL;

	NumNormals = 0;
	if (Normals != NULL)
		delete[] Normals;
	Normals = NULL;

	NumTextureCoordinates = 0;
	if (TextureCoordinates != NULL)
		delete[] TextureCoordinates;
	TextureCoordinates = NULL;

	NumTextures = 0;

	NumVertices = 0;
	if (Vertices != NULL)
		delete[] Vertices;
	Vertices = NULL;

	NumPatches = 0;
	if (Patches != NULL)
		delete[] Patches;
	Patches = NULL;
}

CXGLTexture::CXGLTexture()
{
	TextureBufferHandle = NULL;
	UseTextureBorderColor = 0;
	TextureFunction = 0xffffffff;
	TextureWrap = 0xffffffff;
}

CXGLTexture::~CXGLTexture()
{
	TextureBufferHandle = NULL;
}

CXGLTextureBuffer::CXGLTextureBuffer()
{
	RGBABuffer = NULL;
}

CXGLTextureBuffer::~CXGLTextureBuffer()
{
	if (RGBABuffer != NULL)
		delete [] RGBABuffer;
	RGBABuffer = NULL;
}

CXGLData::CXGLData()
{
	Organization = NULL;
	
	NumStrings = 0;
	StringNames = NULL;
	Strings = NULL;

	NumBins = 0;
	BinNames = NULL;
	Bins = NULL;
	BinLengths = NULL;
}

CXGLData::~CXGLData()
{
	if (Organization != NULL)
		delete[] Organization;
	Organization = NULL;

	unsigned int i;
	for (i = 0; i < NumStrings; i++)
	{
		if (StringNames[i] != NULL)
			delete[] StringNames[i];
		StringNames[i] = NULL;

		if (Strings[i] != NULL)
			delete[] Strings[i];
		Strings[i] = NULL;
	}
	NumStrings = 0;

	for (i = 0; i < NumBins; i++)
	{
		if (BinNames[i] != NULL)
			delete[] BinNames[i];
		BinNames[i] = NULL;

		if (Bins[i] != NULL)
			delete[] Bins[i];
		Bins[i] = NULL;
	}
	NumBins = 0;

	if (StringNames != NULL)
		delete[] StringNames;	
	StringNames = NULL;

	if (Strings != NULL)
		delete[] Strings;
	Strings = NULL;

	if (BinNames != NULL)
		delete[] BinNames;
	BinNames = NULL;

	if (Bins != NULL)
		delete[] Bins;
	Bins = NULL;

	if (BinLengths != NULL)
		delete[] BinLengths;
	BinLengths = NULL;
}

CXGLPatch::CXGLPatch()
{
	NumDatas = 0;
}

CXGLPatch::~CXGLPatch()
{
	NumDatas = 0;
}

CXGLFaceVertex::CXGLFaceVertex()
{
	NormalI = -1;
	VertexI = -1;
	TextureCoordinateI = -1;
}

CXGLFace::CXGLFace()
{
	MaterialI = -1;
	TextureI = -1;
	PatchI = -1;
	SmoothingGroups = 0;
}

CXGLLine::CXGLLine()
{
	MaterialI = -1;
	TextureI = -1;
	PatchI = -1;
	LineStyleI = -1;
}

CXGLPoint::CXGLPoint()
{
	MaterialI = -1;
	TextureI = -1;
	PatchI = -1;
	PointStyleI = -1;
}

CXGLExtgeom::CXGLExtgeom()
{
	Organization = NULL;
	
}

CXGLExtgeom::~CXGLExtgeom()
{
	if (Organization != NULL) delete [] Organization;
	Organization = NULL;
    int numElem = Extgeomelements.GetNumElements();
	if(numElem <= 0) return;
	for(int i = 0; i < numElem; i++)
	{
		delete Extgeomelements.GetElement(i);
	}

}

CXGLExtgeomelem::CXGLExtgeomelem()
{
	Type = NULL;
	Extgeomelemdata = NULL;
	Extgeomelemextents = NULL;
	Extgeomelempriority = -1.;
}

CXGLExtgeomelem::~CXGLExtgeomelem()
{
	if (Type != NULL) delete [] Type;
	if (Extgeomelemdata != NULL) delete [] Extgeomelemdata;
	if (Extgeomelemextents != NULL) delete Extgeomelemextents;
}

int
CXGLObject::GetExtgeomCallbackHandle(int index)
{
	CXGLExtgeomHandle *handle = ExtgeomHandles.GetElement(index);
	return (int)(handle);	
}

int
CXGLMesh::GetExtgeomCallbackHandle(int index)
{
	CXGLExtgeomHandle *handle = ExtgeomHandles.GetElement(index);
	return (int)(handle);	
}
