#ifndef __XGLTypes_h
#define __XGLTypes_h

#include "Utility/ResizeableArray.h"

class CXGLContext;
class CXGLObjectHandle;
class CXGLDataHandle;
class CXGLExtgeomHandle;
class CXGLMeshHandle;
class CXGLTextureHandle;
class CXGLMaterialHandle;
class CXGLVertexHandle;
class CXGLNormalHandle;
class CXGLTextureBufferHandle;

//texture functions
enum
{
	TextureReplace = 0,
	TextureModulate = 1,
	TextureDecal = 2
};

//texture wrap 
enum
{
	TextureRepeat = 0,
	TextureClamp = 1
};

//texture buffer format
enum
{
	TextureRGBA = 0,
	TextureRGB = 1
};

class CXGLTextureBuffer
{
public:

	CXGLTextureBuffer();
	~CXGLTextureBuffer();
	
	unsigned int TextureBufferFormat;
	unsigned int TextureWidth;
	unsigned int TextureHeight;
	unsigned char *RGBABuffer;
};

class CXGLTexture
{
public:
	
	CXGLTexture();
	~CXGLTexture();

	unsigned int TextureFunction;
	unsigned int TextureWrap;
	int UseTextureBorderColor;
	float TextureBorderColor[4];
	
	unsigned int TextureBufferFormat;
	CXGLTextureBufferHandle *TextureBufferHandle;
	int GetTextureBufferCallbackHandle();
};

class CXGLBackground
{
public:
	float Red;
	float Green;
	float Blue;
};

class CXGLTransform
{
public:

	double FX;
	double FY;
	double FZ;
	double UX;
	double UY;
	double UZ;
	double XP;
	double YP;
	double ZP;
	double Scale;
};

class CXGLNormal
{
public:
	int XGLID;
	float X;
	float Y;
	float Z;

	int DefinedXGLID; //For use by CXGLContext only
};

class CXGLVertex
{
public:

	int XGLID;
	
	float X;
	float Y;
	float Z;

	int DefinedXGLID; //For use by CXGLContext only
};

class CXGLTextureCoordinate
{
public:

	int XGLID;

	double U;
	double V;

	int DefinedXGLID; //For use by CXGLContext only
};

class CXGLMaterial
{
public:
	int XGLID;
		
	float AmbientR;
	float AmbientG;
	float AmbientB;
	float DiffuseR;
	float DiffuseG;
	float DiffuseB;
	float SpecularR;
	float SpecularG;
	float SpecularB;
	float Shininess;
	float Alpha;
	float EmissiveR;
	float EmissiveG;
	float EmissiveB;

	int DefinedXGLID; //For use by CXGLContext only

};

class CXGLLineStyle
{
public:
	int XGLID;

	float Width;
	unsigned short Pattern;
	unsigned int PatternFactor;

	int DefinedXGLID; //For use by CXGLContext only
};

class CXGLPointStyle
{
public:
	int XGLID;

	float Size;

	int DefinedXGLID; //For use by CXGLContext only
};

class CXGLFaceVertex
{	
public:

	CXGLFaceVertex();

	unsigned int VertexI;
	unsigned int NormalI;
	unsigned int TextureCoordinateI;
};

class CXGLFace
{
public:

	CXGLFace();

	CXGLFaceVertex FaceVertices[3];
	unsigned int TextureI;
	unsigned int MaterialI;
	unsigned int PatchI;
	unsigned int SmoothingGroups;
};

class CXGLLine
{
public:

	CXGLLine();

	CXGLFaceVertex LineVertices[2];
	unsigned int TextureI;
	unsigned int MaterialI;
	unsigned int PatchI;
	unsigned int LineStyleI;
};

class CXGLPoint
{
public:

	CXGLPoint();

	CXGLFaceVertex PointVertex;
	unsigned int TextureI;
	unsigned int MaterialI;
	unsigned int PatchI;
	unsigned int PointStyleI;
};

class CXGLPatch
{
public:
	CXGLPatch();
	~CXGLPatch();

	unsigned int PatchID;

	unsigned int NumDatas;
	CResizeableArray<CXGLDataHandle*> DataHandles;

	int GetDataCallbackHandle(int index);
};

class CXGLMesh
{
public:

	CXGLMesh();
	~CXGLMesh();

	int DoubleSided;

	unsigned int NumFaces;
	CXGLFace *Faces;

	unsigned int NumLines;
	CXGLLine *Lines;

	unsigned int NumPoints;
	CXGLPoint *Points;

	unsigned int NumMaterials;
	CXGLMaterial *Materials;

	unsigned int NumLineStyles;
	CXGLLineStyle *LineStyles;

	unsigned int NumPointStyles;
	CXGLPointStyle *PointStyles;

	unsigned int NumTextures;
	CResizeableArray<CXGLTextureHandle*> TextureHandles;

	unsigned int NumVertices;
	CXGLVertex *Vertices;

	unsigned int NumNormals;
	CXGLNormal *Normals;

	unsigned int NumTextureCoordinates;
	CXGLTextureCoordinate* TextureCoordinates;

	unsigned int NumPatches;
	CXGLPatch* Patches;

	unsigned int NumExtgeoms;
	CResizeableArray<CXGLExtgeomHandle*> ExtgeomHandles;

	int GetTextureCallbackHandle(int index);
	int GetExtgeomCallbackHandle(int index);
};

class CXGLExtents
{
public:

	double MinX;
	double MinY;
	double MinZ;

	double MaxX;
	double MaxY;
	double MaxZ;
};



class CXGLObject
{
public:
	CXGLObject();
	~CXGLObject();

	int IsExternal;
	int IsInResolveLoop;    //Used only by XGLResolver.cpp
	int XGLID;				//Used only by XGLImport.cpp
	int PathID;			    //Should be set by importer

	char *Name;

	CXGLTransform Transform;

	CXGLMeshHandle *MeshHandle;	

	unsigned int NumDatas;
	unsigned int NumObjects;
	unsigned int NumExtgeoms;
	CResizeableArray<CXGLDataHandle*> DataHandles;
	CResizeableArray<CXGLObjectHandle*> ObjectHandles;
	CResizeableArray<CXGLExtgeomHandle*> ExtgeomHandles;

	CXGLExtents Extents;
	char* Ref;
	char* RefType;
	int IsStatic;

	int GetObjectCallbackHandle(int index);
	int GetDataCallbackHandle(int index);
	int GetMeshCallbackHandle();
	int GetExtgeomCallbackHandle(int index);

protected:
	NumReferences();
};

class CXGLLight
{
public:
	float DiffuseR;
	float DiffuseG;
	float DiffuseB;
	float SpecularR;
	float SpecularG;
	float SpecularB;
	float DX;
	float DY;
	float DZ;
};

class CXGLLighting
{
public:

	float AmbientR;
	float AmbientG;
	float AmbientB;

	int NumLights;

	CXGLLight Lights[8];
};

class CXGLWorld
{
public:

	CXGLWorld();
	~CXGLWorld();

	unsigned int NumDatas;
	unsigned int NumObjects;
	CResizeableArray<CXGLDataHandle*> DataHandles;
	CResizeableArray<CXGLObjectHandle*> ObjectHandles;

	char *Name;

	CXGLMeshHandle *MeshHandle;	

	int GetObjectCallbackHandle(int index);
	int GetDataCallbackHandle(int index);
	int GetMeshCallbackHandle();
};

class CXGLData
{
public:
	CXGLData();
	~CXGLData();

	char *Organization;

	unsigned int NumStrings;
	char **StringNames;
	char **Strings;

	unsigned int NumBins;
	char **BinNames;
	unsigned int *BinLengths;
	unsigned char **Bins;
};

// Exdended Geometry XGL classes

class CXGLExtgeomelem
{
public:
	CXGLExtgeomelem();
	~CXGLExtgeomelem();
	char *Type;
	unsigned int LengthExtgeomelemdata;
	unsigned char *Extgeomelemdata;
	CXGLExtents *Extgeomelemextents;
	float Extgeomelempriority;
};

class CXGLExtgeom
{
public:
	CXGLExtgeom();
	~CXGLExtgeom();
	char *Organization;
	CResizeableArray<CXGLExtgeomelem*> Extgeomelements;
};

#endif