#ifndef __ZLIBCompress_h
#define __ZLIBCompress_h

#include <stdio.h>
#include "ZLIB.h"

#ifdef WIN32
#include <windows.h>
#endif

class CZLIBBufferCompress
{
public:

	bool Initialize(const char *zlibModulePath);
	bool UnInitialize();

	bool SetData(const char *data);
	bool SetData(const char *data, unsigned int length);

	bool ReleaseData();

	//dataToFill for GetCompressedData & GetCompressedDataNoFlush
	//must be 100.1%*(original data) + 12 --from zlib spec.

	// flushes automatically
	bool GetCompressedData(char *dataToFill, unsigned int *bytesRead);

	// can be called multiple times
	bool GetCompressedDataNoFlush(char *dataToFill, unsigned int *bytesRead);
	
	// call at the end of a sequence of calls to GetCompressedDataNoFlush in order to complete the compression
	bool Finish(char *dataToFill, unsigned int *bytesRead);
	
	// resets the compressor to begin a new compression
	bool Reset();

	char *GetLastErrorString();

private:

	bool Error;
	char ErrorBuffer[1024];
	void SetError(char *errorString);
	void SetZLibError(char *errorString, int zlibError);

#ifdef WIN32
	HMODULE DllHandle;
#endif

	z_stream ZStream;
	char *UnCompressedBuffer;
	unsigned int UnCompressedBufferLength;
	unsigned char CompressedBuffer[65536];

	//Our Zlib Functions
	int (__stdcall *deflateInit_) (z_stream *strm, int level, const char *version, int stream_size);
	int (__stdcall *deflate) (z_stream *strm, int flush);
	int (__stdcall *deflateEnd) (z_stream *strm);
	int (__stdcall *deflateReset) (z_stream *strm);
};

class CZLIBFileCompress
{
public:

	bool Initialize(const char *zlibModulePath, const char *fileName);
	bool Initialize(const char *zlibModulePath, FILE *file);
	bool UnInitialize();

	bool OutputData(const char *data);
	bool OutputData(const char *data, unsigned int length);

	bool Finish();

	char *GetLastErrorString();

private:

	bool Initialize(const char *zlibModulePath);

	bool Error;
	char ErrorBuffer[1024];
	void SetError(char *errorString);
	void SetErrorEx(char *errorString);
	void SetZLibError(char *errorString, int zlibError);

#ifdef WIN32
	HINSTANCE DllHandle;
#endif

	FILE *Stream;
	bool IsStreamUserOwned;
	z_stream ZStream;
	unsigned char CompressedBuffer[65536];

	//Our Zlib Functions
	int (__stdcall *deflateInit_) (z_stream *strm, int level, const char *version, int stream_size);
	int (__stdcall *deflate) (z_stream *strm, int flush);
	int (__stdcall *deflateEnd) (z_stream *strm);
};

#endif