// ChildView.cpp : implementation of the CChildView class
//

#include "stdafx.h"
#include "XGLViewer.h"
#include "ChildView.h"
#include "OpenGL.h"
#include "math.h"
#include "XGLState.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CChildView

CChildView::CChildView()
{
}

CChildView::~CChildView()
{
}


BEGIN_MESSAGE_MAP(CChildView,CWnd )
	//{{Afx_MSG_MAP(CChildView)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	//}}Afx_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW+1), NULL);

	return TRUE;
}

void CChildView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	
	//the window needs to repaint so we should draw the world if there is one
	CallDisplayList();
	// Do not call CWnd::OnPaint() for painting messages
}

void CChildView::OnSize(UINT nType, int cx, int cy) 
{
	CWnd ::OnSize(nType, cx, cy);

	//tell opengl that the physical size of our window has changed
	SetViewPort(cx, cy);

	//these two calls are very ugly and should be changed
	//this effectly tells opengl to reset it's view frustum because 
	//the size of the window has changed
	double x, y, z, xw, yw, zw;
	XGLState.GetWindowCoords(&x, &y, &z, &xw, &yw, &zw);
	XGLState.SetWindowCoords(x, y, z, xw, yw, zw);

	//redraw the world
	CallDisplayList();
}

int CChildView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd ::OnCreate(lpCreateStruct) == -1)
		return -1;

	IsPressed = 0;
	XGLState.SetWorldRotation(0.0, 0.0, 1.0, 0.0, 1.0, 0.0);
	
	GLInit(m_hWnd);
	
	return 0;
}

void CChildView::OnDestroy()
{
	GLUnInit();
	CWnd ::OnDestroy();
}



void CChildView::OnMouseMove(UINT nFlags, CPoint point) 
{
	//this function determines how to manipulate the world 
	//depending on our mode and whether we are clicking or dragging, etc.
	if (IsPressed != 0)
	{
		switch (LeftButtonMode)
		{
		case MouseModeExamine:
			{
			
				//this case rotates our world around
			signed int xPos = (signed short) point.x;  
			signed int yPos = (signed short) point.y;  
			
			if (HasMoved == 0)
			{
				if ((abs(xPos - XPos) < 3) && (abs(yPos-YPos) < 3))
					return;
			}

			if ((PrevXPos == xPos) && (PrevYPos == yPos))
				return;

			
			double x = (PrevXPos-xPos) * 0.015;
			double y = (yPos - PrevYPos) * 0.015;
			
			double r = sqrt(x*x + y*y);
			double ir = 1.0/r;

			double dx = x * ir;
			double dy = y * ir;

			double s = sin(r);
			double c = cos(r);

			double fx, fy, fz, ux, uy, uz;
			XGLState.GetWorldRotation(&fx, &fy, &fz, &ux, &uy, &uz);
			fx = -fx;
			ux = -ux;

			double fx1 = dy*fx + dx*fy;
			double fy1 = -dx*fx + dy*fy;
			double fz1 = fz;
			
			double fx2 = fx1;
			double fy2 =  c*fy1 - s*fz1;
			double fz2 =  s*fy1 + c*fz1;

			fx =  dy*fx2 - dx*fy2;
			fy = +dx*fx2 + dy*fy2;
			fz = fz2;

			double imag = 1.0 / (fx*fx + fy*fy + fz*fz);
			fx*=imag; fy*=imag; fz*=imag;

			double ux1 = dy*ux + dx*uy;
			double uy1 = -dx*ux + dy*uy;
			double uz1 = uz;
			
			double ux2 = ux1;
			double uy2 =  c*uy1 - s*uz1;
			double uz2 =  s*uy1 + c*uz1;

			ux =  dy*ux2 - dx*uy2;
			uy = +dx*ux2 + dy*uy2;
			uz = uz2;

			
			double err = ux*fx + uy*fy + uz*fz;
			ux = ux - fx * err;
			uy = uy - fy * err;
			uz = uz - fz * err;
			

			imag = 1.0 / (ux*ux + uy*uy + uz*uz);
			ux*=imag; uy*=imag; uz*=imag;

			PrevXPos = xPos;
			PrevYPos = yPos;

			XPos = xPos;
			YPos = yPos;
		
			HasMoved = 1;

			XGLState.SetWorldRotation(-fx, fy, fz, -ux, uy, uz);
			break;
			}
		case MouseModePan:
			{
				//this pans our world around
				RECT r;
				GetClientRect(&r);
				double width = r.right - r.left;
				double height = r.bottom - r.top;
				double px = point.x;
				double py = point.y;

				double xp = -(((px-StartXPos)/width))*(width/height);
				double yp = ((py-StartYPos)/height);

				double x, y, z, xw, yw, zw;
				XGLState.GetWindowCoords(&x, &y, &z, &xw, &yw, &zw);

				double inputX, inputY;
				//if we've panned past 100 times the size of the world, then there's no need to continue
				if (fabs(xp*xw + InitWindowX) > 100*zw)
					inputX = 100*zw;
				else
					inputX = xp*xw + InitWindowX;

				if (fabs(yp*yw + InitWindowY) > 100*zw)
					inputY = 10*zw;
				else
					inputY = yp*yw + InitWindowY;

				XGLState.SetWindowCoords(inputX, inputY, 0, xw, yw, zw);

			}
			break;
		case MouseModeZoomIn:
			{
				//this case zooms us in and out when we drag up and down
				RECT r;
				GetClientRect(&r);

				double py = point.y;
				double zoomfactor = 1 - (2*(point.y - StartYPos)/((double)(r.bottom - r.top)));
			
				double x, y, z, xw, yw, zw;
				XGLState.GetWindowCoords(&x, &y, &z, &xw, &yw, &zw);

				// if we have zoomed out 50 times the size of the world
				// or if we've zoomed in 1/5000th the size of the world then quit.
				double inputWidth;
				if (InitWindowWidth*zoomfactor > 50*zw)
				{
					inputWidth = 50*zw;
				}
				else if (InitWindowWidth*zoomfactor < 0.0005*zw)
				{
					inputWidth = 0.0005*zw;
				}
				else
				{
					inputWidth = InitWindowWidth*zoomfactor;
				}

				XGLState.SetWindowCoords(x, y, z, inputWidth, inputWidth, zw);
			}
			break;
		default:
			break;
		}
	}
}

void CChildView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	signed int x = (signed short) point.x;
	signed int y = (signed short) point.y;

//	double xRel = ((double) x - CenterX) / RadiusX;
	//double yRel = ((double) y - CenterY) / RadiusY;

	//if ((xRel*xRel + yRel*yRel) > 1.0)
	//	return;

	int xDiff = StartXPos - XPos;
	int yDiff = StartYPos - YPos;

	StartXPos = x;  
	StartYPos = y;

	XPos = StartXPos - xDiff;
	YPos = StartYPos - yDiff;

	PrevXPos = x;
	PrevYPos = y;

	IsPressed = 1;
	HasMoved = 0;

	double xp, yp, zp, xw, yw, zw;
	XGLState.GetWindowCoords(&xp, &yp, &zp, &xw, &yw, &zw);
	InitWindowWidth = xw;
	InitWindowX = xp;
	InitWindowY = yp;
	SetCapture();
}

void CChildView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	ReleaseCapture();
	IsPressed = 0;
}

void 
CChildView::SetWindowCursor(HCURSOR c)
{
	SetClassLong(m_hWnd, GCL_HCURSOR, (LONG) c);
}

void 
CChildView::UpdateCursor()
{
	//we need to possibly change the cursor due to a mousemode state change
	int resourceID = 0;

	switch (LeftButtonMode)
	{
	case MouseModeExamine:
		resourceID = IDC_ExamineCursor;
		break;
	case MouseModeZoomIn:
		resourceID = IDC_ZoomInCursor;
		break;
	case MouseModePan:
		resourceID = IDC_PanCursor;
		break;
	default:
		//this gives you a default cursor
		SetWindowCursor(NULL);
		break;
	}
	
	if (resourceID != 0)
	{
		HCURSOR c = AfxGetApp()->LoadCursor(MAKEINTRESOURCE(resourceID));
		SetWindowCursor(c);
	}

}

void 
CChildView::SetMouseMode(int mouseMode)
{
	//set the mouse mode and update the cursor
	LeftButtonMode = mouseMode;
	UpdateCursor();
}


void 
CChildView::ZoomOut()
{
	//this function acts as sort of a home button, resetting all
	//of view variables so that the world looks just as it did 
	//when we first opened the file.
	double x, y, z, xw, yw, zw;
	XGLState.GetWindowCoords(&x, &y, &z, &xw, &yw, &zw);
	XGLState.SetWindowCoords(0, 0, 0, zw, zw, zw);
	XGLState.SetWorldRotation(0.0, 0.0, 1.0, 0.0, 1.0, 0.0);
}