#ifndef __Import_h
#define __Import_h

#include "XGLImport/XGLImportCallback.h"
#include "XGLImport/XGLTypes.h"
#include "Utility/ResizeableArray.h"

#define MAX_TEXTURE_DIM_PIXELS 4096
#define MIN_TEXTURE_DIM_PIXELS 32

class CDisplayListNode
{
public:
	CDisplayListNode()
	{
		NormalFaces = 0;
		TextureFaces = 0;
		AlphaTextureFaces = 0;
		AlphaFaces = 0;
	}

	unsigned int NormalFaces;
	unsigned int TextureFaces;
	unsigned int AlphaFaces;
	unsigned int AlphaTextureFaces;

	unsigned int ExtentsIndex;
};

class CImport : public CXGLImportCallback
{
public:

	CImport();
	~CImport();

	void ImportFile(const char *fileName);

	//These functions implement the callbacks from 
	//callbacks
	virtual void WorldCallback(CXGLWorld* world);
	virtual int MeshCallback(CXGLMesh* mesh);
	virtual int ObjectCallback(CXGLObject *object);
	virtual int DataCallback(CXGLData* data);
	virtual int TextureCallback(CXGLTexture* texture);
	virtual int TextureBufferCallback(CXGLTextureBuffer *textureBuffer);
	virtual void ErrorCallback(const char* errorString);
	virtual void BackgroundCallback(CXGLBackground* background);
	virtual void LightingCallback(CXGLLighting* lighting);
	virtual int ExtgeomCallback(CXGLExtgeom* extgeom);
	
	CXGLWorld *World;
	CXGLLighting *Lighting;
	CXGLBackground *Background;

	CResizeableArray<CDisplayListNode> DisplayListNodes;

protected:

	void CalculateWorldSize();
	bool CalculateObjectSize(CXGLObject *object, double *objectMinX, double *objectMinY, double *objectMinZ, double *objectMaxX, double *objectMaxY, double *objectMaxZ);
	void CalculateMeshSize(CXGLMesh *mesh, double *meshMinX, double *meshMinY, double *meshMinZ, double *meshMaxX, double *meshMaxY, double *meshMaxZ);
	void CalculateFaceCounts(CXGLMesh *mesh, unsigned int *numNormalFaces, unsigned int *numTextureFaces, unsigned int *numAlphaTextureFaces, unsigned int *numAlphaFaces);
	void DeletePreviousData();
	
	double WorldX, WorldY, WorldZ;
	double WorldExtent;

	//resizeable array to hold the size of our objects
	CResizeableArray<double> ObjectMinX;
	CResizeableArray<double> ObjectMinY;
	CResizeableArray<double> ObjectMinZ;
	CResizeableArray<double> ObjectMaxX;
	CResizeableArray<double> ObjectMaxY;
	CResizeableArray<double> ObjectMaxZ;

	//resizeable array to hold the size of our meshes
	CResizeableArray<double> MeshMinX;
	CResizeableArray<double> MeshMinY;
	CResizeableArray<double> MeshMinZ;
	CResizeableArray<double> MeshMaxX;
	CResizeableArray<double> MeshMaxY;
	CResizeableArray<double> MeshMaxZ;
};

extern CImport Import;




#endif