// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "XGLViewer.h"
#include "OpenGL.h"

#include "MainFrm.h"
#include "Import.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_TOOLS_PAN, OnToolsPan)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_PAN, OnUpdateToolsPan)
	ON_COMMAND(ID_TOOLS_EXAMINE, OnToolsExamine)
	ON_COMMAND(ID_TOOLS_ZOOMIN, OnToolsZoomin)
	ON_COMMAND(ID_TOOLS_ZOOMOUT, OnToolsZoomout)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_EXAMINE, OnUpdateToolsExamine)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_ZOOMIN, OnUpdateToolsZoomin)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_ZOOMOUT, OnUpdateToolsZoomout)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE, OnUpdateFileClose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	// create a view to occupy the client area of the frame
	if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW,
		CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	//the timer
	SetTimer(0, 50, NULL);

	//Set view mode
	LeftButtonMode = MouseModeExamine;
	m_wndView.SetMouseMode(LeftButtonMode);

	IsOpenFile = 0;

	// This part of code permits to input file name from command line
	// or by double-clicking on the file name in the folder
	if (__argc > 1)
	{
		char *filename = __argv[1];

		HCURSOR oldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
		Import.ImportFile(filename);
		SetCursor(oldCursor);

		char buffer[1024];
		char *text = buffer;
		if (strlen(filename) < 1024)
		{
			strcpy(text, filename);
			strcat(text, " - ");
		}
		strcat(text, "XGLViewer");
		SetWindowText(text);

		IsOpenFile = 1;
	}

	return 0;
}

void CMainFrame::OnDestroy()
{
	m_wndToolBar.DestroyWindow();
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::OnSetFocus(CWnd* pOldWnd)
{
	// forward focus to the view window
	m_wndView.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}


void CMainFrame::OnFileOpen() 
{
	/*
	This function spawns an open dialog box from which the user chooses an xgl file to view.  
	Then we start the importer on the chosen file.
	*/

	static char szFilter[] = "XGL Files (*.xgl; *.zgl)|*.xgl; *.zgl|All Files (*.*)|*.*||";

	CFileDialog openDialog(TRUE, NULL, NULL, OFN_HIDEREADONLY, szFilter, this);
	openDialog.DoModal();
	CString pathName = openDialog.GetPathName();
	if (strcmp(pathName, "") != 0)
	{
		HCURSOR oldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
		Import.ImportFile(pathName);
		SetCursor(oldCursor);
		char buffer[1024];
		char *text = buffer;
		if (strlen(pathName) < 1024)
		{
			strcpy(text, pathName);
			strcat(text, " - ");
		}
		strcat(text, "XGLViewer");
		SetWindowText(text);
		IsOpenFile = 1;
	}
}

//the button or menu callbacks are used to switch cursors and state in the childview
void CMainFrame::OnToolsExamine() 
{
	if (LeftButtonMode == MouseModeExamine)
		LeftButtonMode = MouseModeNone;
	else
		LeftButtonMode = MouseModeExamine;

	m_wndView.SetMouseMode(LeftButtonMode);
}

void CMainFrame::OnToolsPan() 
{
	if (LeftButtonMode == MouseModePan)
		LeftButtonMode = MouseModeNone;
	else
		LeftButtonMode = MouseModePan;

	m_wndView.SetMouseMode(LeftButtonMode);
}

void CMainFrame::OnToolsZoomin() 
{
	if (LeftButtonMode == MouseModeZoomIn)
		LeftButtonMode = MouseModeNone;
	else
		LeftButtonMode = MouseModeZoomIn;

	m_wndView.SetMouseMode(LeftButtonMode);
}

void CMainFrame::OnToolsZoomout() 
{
	//zoom out
	m_wndView.ZoomOut();
}


//update ui button state based on our mousemode
void CMainFrame::OnUpdateToolsExamine(CCmdUI* pCmdUI) 
{
	if (LeftButtonMode == MouseModeExamine)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainFrame::OnUpdateToolsPan(CCmdUI* pCmdUI)
{
	if (LeftButtonMode == MouseModePan)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainFrame::OnUpdateToolsZoomin(CCmdUI* pCmdUI) 
{
	if (LeftButtonMode == MouseModeZoomIn)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainFrame::OnUpdateToolsZoomout(CCmdUI* pCmdUI) 
{
	if (LeftButtonMode == MouseModeZoomOut)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}


void CMainFrame::OnFileClose() 
{
	//when we close the file we need to delete the display lists (if any) created when we imported the xgl
	//file. 
	DeleteDisplayLists();

	//reset the title bar
	SetWindowText("XGLViewer");

	//mark the viewer as having no open file
	IsOpenFile = 0;
}


void CMainFrame::OnUpdateFileClose(CCmdUI* pCmdUI) 
{
	//determine if we should enable the close menu option
	if (IsOpenFile == 1)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}
