#include "stdafx.h"
#include "OpenGL.h"
#include "gl/glu.h"
#include "XGLImport/XGLTypes.h"
#include "Import.h"

#include <assert.h>
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//the window, dc and rc we're rendering to.
HDC DC;
HGLRC RC;
HWND Window;

//an identifier for our world display list
GLuint WorldID = 0;

//an integer representing the upper bound of
//the range of our display lists
unsigned int DisplayListRange = 1;

//our world transform
float UX = 0.0;
float UY = 1.0;
float UZ = 0.0;
float FX = 0.0;
float FY = 0.0;
float FZ = 1.0;
float XP = 0.0;
float YP = 0.0;
float ZP = 0.0;


//this is a palette for 256-color mode "yuck"
HPALETTE RGBPalette;
static struct {
    WORD        palVersion;
    WORD        palNumEntries;
    PALETTEENTRY palPalEntries[256];
} rgb8Palette = {
    0x300,
    256,
      0,   0,   0,   0,    /* 0 - Exact match with default 0 */
     63,   0,   0,   0,    /* 1 */
    104,   0,   0,   0,    /* 2 */
    128,   0,   0,   0,    /* 3 - Changed to match default 1 */
    171,   0,   0,   0,    /* 4 */
    200,   0,   0,   0,    /* 5 */
    229,   0,   0,   0,    /* 6 */
    255,   0,   0,   0,    /* 7 - Exact match with default 13 */
      0,  63,   0,   0,    /* 8 */
     63,  63,   0,   0,    /* 9 */
    104,  63,   0,   0,    /* 10 */
    139,  63,   0,   0,    /* 11 */
    171,  63,   0,   0,    /* 12 */
    200,  63,   0,   0,    /* 13 */
    229,  63,   0,   0,    /* 14 */
    255,  63,   0,   0,    /* 15 */
      0, 104,   0,   0,    /* 16 */
     63, 104,   0,   0,    /* 17 */
    104, 104,   0,   0,    /* 18 */
    139, 104,   0,   0,    /* 19 */
    171, 104,   0,   0,    /* 20 */
    200, 104,   0,   0,    /* 21 */
    229, 104,   0,   0,    /* 22 */
    255, 104,   0,   0,    /* 23 */
      0, 128,   0,   0,    /* 24 - Changed to match default 2 */
     63, 139,   0,   0,    /* 25 */
    104, 139,   0,   0,    /* 26 */
    128, 128,   0,   0,    /* 27 - Changed to match default 3 */
    171, 139,   0,   0,    /* 28 */
    200, 139,   0,   0,    /* 29 */
    229, 139,   0,   0,    /* 30 */
    255, 139,   0,   0,    /* 31 */
      0, 171,   0,   0,    /* 32 */
     63, 171,   0,   0,    /* 33 */
    104, 171,   0,   0,    /* 34 */
    139, 171,   0,   0,    /* 35 */
    171, 171,   0,   0,    /* 36 */
    200, 171,   0,   0,    /* 37 */
    229, 171,   0,   0,    /* 38 */
    255, 171,   0,   0,    /* 39 */
      0, 200,   0,   0,    /* 40 */
     63, 200,   0,   0,    /* 41 */
    104, 200,   0,   0,    /* 42 */
    139, 200,   0,   0,    /* 43 */
    171, 200,   0,   0,    /* 44 */
    200, 200,   0,   0,    /* 45 */
    229, 200,   0,   0,    /* 46 */
    255, 200,   0,   0,    /* 47 */
      0, 229,   0,   0,    /* 48 */
     63, 229,   0,   0,    /* 49 */
    104, 229,   0,   0,    /* 50 */
    139, 229,   0,   0,    /* 51 */
    171, 229,   0,   0,    /* 52 */
    200, 229,   0,   0,    /* 53 */
    229, 229,   0,   0,    /* 54 */
    255, 229,   0,   0,    /* 55 */
      0, 255,   0,   0,    /* 56 - Exact match with default 14 */
     63, 255,   0,   0,    /* 57 */
    104, 255,   0,   0,    /* 58 */
    139, 255,   0,   0,    /* 59 */
    171, 255,   0,   0,    /* 60 */
    200, 255,   0,   0,    /* 61 */
    229, 255,   0,   0,    /* 62 */
    255, 255,   0,   0,    /* 63 - Exact match with default 15 */
      0,   0, 128,   0,    /* 64 - Changed to match default 4 */
     63,   0, 116,   0,    /* 65 */
    104,   0, 116,   0,    /* 66 */
    128,   0, 128,   0,    /* 67 - Changed to match default 5 */
    171,   0, 116,   0,    /* 68 */
    200,   0, 116,   0,    /* 69 */
    229,   0, 116,   0,    /* 70 */
    255,   0, 116,   0,    /* 71 */
      0,  63, 116,   0,    /* 72 */
     63,  63, 116,   0,    /* 73 */
    104,  63, 116,   0,    /* 74 */
    139,  63, 116,   0,    /* 75 */
    171,  63, 116,   0,    /* 76 */
    200,  63, 116,   0,    /* 77 */
    229,  63, 116,   0,    /* 78 */
    255,  63, 116,   0,    /* 79 */
      0, 104, 116,   0,    /* 80 */
     63, 104, 116,   0,    /* 81 */
    104, 104, 116,   0,    /* 82 */
    139, 104, 116,   0,    /* 83 */
    171, 104, 116,   0,    /* 84 */
    200, 104, 116,   0,    /* 85 */
    229, 104, 116,   0,    /* 86 */
    255, 104, 116,   0,    /* 87 */
      0, 128, 128,   0,    /* 88 - Changed to match default 6 */
     63, 139, 116,   0,    /* 89 */
    104, 139, 116,   0,    /* 90 */
    128, 128, 128,   0,    /* 91 - Changed to match default 12 */
    171, 139, 116,   0,    /* 92 */
    200, 139, 116,   0,    /* 93 */
    229, 139, 116,   0,    /* 94 */
    255, 139, 116,   0,    /* 95 */
      0, 171, 116,   0,    /* 96 */
     63, 171, 116,   0,    /* 97 */
    104, 171, 116,   0,    /* 98 */
    139, 171, 116,   0,    /* 99 */
    171, 171, 116,   0,    /* 100 */
    200, 171, 116,   0,    /* 101 */
    229, 171, 116,   0,    /* 102 */
    255, 171, 116,   0,    /* 103 */
      0, 200, 116,   0,    /* 104 */
     63, 200, 116,   0,    /* 105 */
    104, 200, 116,   0,    /* 106 */
    139, 200, 116,   0,    /* 107 */
    171, 200, 116,   0,    /* 108 */
    200, 200, 116,   0,    /* 109 */
    229, 200, 116,   0,    /* 110 */
    255, 200, 116,   0,    /* 111 */
      0, 229, 116,   0,    /* 112 */
     63, 229, 116,   0,    /* 113 */
    104, 229, 116,   0,    /* 114 */
    139, 229, 116,   0,    /* 115 */
    171, 229, 116,   0,    /* 116 */
    200, 229, 116,   0,    /* 117 */
    229, 229, 116,   0,    /* 118 */
    255, 229, 116,   0,    /* 119 */
      0, 255, 116,   0,    /* 120 */
     63, 255, 116,   0,    /* 121 */
    104, 255, 116,   0,    /* 122 */
    139, 255, 116,   0,    /* 123 */
    171, 255, 116,   0,    /* 124 */
    200, 255, 116,   0,    /* 125 */
    229, 255, 116,   0,    /* 126 */
    255, 255, 116,   0,    /* 127 */
      0,   0, 191,   0,    /* 128 */
     63,   0, 191,   0,    /* 129 */
    104,   0, 191,   0,    /* 130 */
    139,   0, 191,   0,    /* 131 */
    171,   0, 191,   0,    /* 132 */
    200,   0, 191,   0,    /* 133 */
    229,   0, 191,   0,    /* 134 */
    255,   0, 191,   0,    /* 135 */
      0,  63, 191,   0,    /* 136 */
     63,  63, 191,   0,    /* 137 */
    104,  63, 191,   0,    /* 138 */
    139,  63, 191,   0,    /* 139 */
    171,  63, 191,   0,    /* 140 */
    200,  63, 191,   0,    /* 141 */
    229,  63, 191,   0,    /* 142 */
    255,  63, 191,   0,    /* 143 */
      0, 104, 191,   0,    /* 144 */
     63, 104, 191,   0,    /* 145 */
    104, 104, 191,   0,    /* 146 */
    139, 104, 191,   0,    /* 147 */
    171, 104, 191,   0,    /* 148 */
    200, 104, 191,   0,    /* 149 */
    229, 104, 191,   0,    /* 150 */
    255, 104, 191,   0,    /* 151 */
      0, 139, 191,   0,    /* 152 */
     63, 139, 191,   0,    /* 153 */
    104, 139, 191,   0,    /* 154 */
    139, 139, 191,   0,    /* 155 */
    171, 139, 191,   0,    /* 156 */
    200, 139, 191,   0,    /* 157 */
    229, 139, 191,   0,    /* 158 */
    255, 139, 191,   0,    /* 159 */
      0, 171, 191,   0,    /* 160 */
     63, 171, 191,   0,    /* 161 */
    104, 171, 191,   0,    /* 162 */
    139, 171, 191,   0,    /* 163 */
    160, 160, 164,   0,    /* 164 - Changed to match default 11 */
    200, 171, 191,   0,    /* 165 */
    229, 171, 191,   0,    /* 166 */
    255, 171, 191,   0,    /* 167 */
      0, 200, 191,   0,    /* 168 */
     63, 200, 191,   0,    /* 169 */
    104, 200, 191,   0,    /* 170 */
    139, 200, 191,   0,    /* 171 */
    171, 200, 191,   0,    /* 172 */
    192, 192, 192,   0,    /* 173 - Changed to match default 7 */
    229, 200, 191,   0,    /* 174 */
    255, 200, 191,   0,    /* 175 */
      0, 229, 191,   0,    /* 176 */
     63, 229, 191,   0,    /* 177 */
    104, 229, 191,   0,    /* 178 */
    139, 229, 191,   0,    /* 179 */
    171, 229, 191,   0,    /* 180 */
    192, 220, 192,   0,    /* 181 - Changed to match default 8 */
    229, 229, 191,   0,    /* 182 */
    255, 229, 191,   0,    /* 183 */
      0, 255, 191,   0,    /* 184 */
     63, 255, 191,   0,    /* 185 */
    104, 255, 191,   0,    /* 186 */
    139, 255, 191,   0,    /* 187 */
    171, 255, 191,   0,    /* 188 */
    200, 255, 191,   0,    /* 189 */
    229, 255, 191,   0,    /* 190 */
    255, 255, 191,   0,    /* 191 */
      0,   0, 255,   0,    /* 192 - Exact match with default 16 */
     63,   0, 255,   0,    /* 193 */
    104,   0, 255,   0,    /* 194 */
    139,   0, 255,   0,    /* 195 */
    171,   0, 255,   0,    /* 196 */
    200,   0, 255,   0,    /* 197 */
    229,   0, 255,   0,    /* 198 */
    255,   0, 255,   0,    /* 199 - Exact match with default 17 */
      0,  63, 255,   0,    /* 200 */
     63,  63, 255,   0,    /* 201 */
    104,  63, 255,   0,    /* 202 */
    139,  63, 255,   0,    /* 203 */
    171,  63, 255,   0,    /* 204 */
    200,  63, 255,   0,    /* 205 */
    229,  63, 255,   0,    /* 206 */
    255,  63, 255,   0,    /* 207 */
      0, 104, 255,   0,    /* 208 */
     63, 104, 255,   0,    /* 209 */
    104, 104, 255,   0,    /* 210 */
    139, 104, 255,   0,    /* 211 */
    171, 104, 255,   0,    /* 212 */
    200, 104, 255,   0,    /* 213 */
    229, 104, 255,   0,    /* 214 */
    255, 104, 255,   0,    /* 215 */
      0, 139, 255,   0,    /* 216 */
     63, 139, 255,   0,    /* 217 */
    104, 139, 255,   0,    /* 218 */
    139, 139, 255,   0,    /* 219 */
    171, 139, 255,   0,    /* 220 */
    200, 139, 255,   0,    /* 221 */
    229, 139, 255,   0,    /* 222 */
    255, 139, 255,   0,    /* 223 */
      0, 171, 255,   0,    /* 224 */
     63, 171, 255,   0,    /* 225 */
    104, 171, 255,   0,    /* 226 */
    139, 171, 255,   0,    /* 227 */
    171, 171, 255,   0,    /* 228 */
    200, 171, 255,   0,    /* 229 */
    229, 171, 255,   0,    /* 230 */
    255, 171, 255,   0,    /* 231 */
      0, 200, 255,   0,    /* 232 */
     63, 200, 255,   0,    /* 233 */
    104, 200, 255,   0,    /* 234 */
    139, 200, 255,   0,    /* 235 */
    166, 202, 240,   0,    /* 236 - Changed to match default 9 */
    200, 200, 255,   0,    /* 237 */
    229, 200, 255,   0,    /* 238 */
    255, 200, 255,   0,    /* 239 */
      0, 229, 255,   0,    /* 240 */
     63, 229, 255,   0,    /* 241 */
    104, 229, 255,   0,    /* 242 */
    139, 229, 255,   0,    /* 243 */
    171, 229, 255,   0,    /* 244 */
    200, 229, 255,   0,    /* 245 */
    229, 229, 255,   0,    /* 246 */
    255, 251, 240,   0,    /* 247 - Changed to match default 10 */
      0, 255, 255,   0,    /* 248 - Exact match with default 18 */
     63, 255, 255,   0,    /* 249 */
    104, 255, 255,   0,    /* 250 */
    139, 255, 255,   0,    /* 251 */
    171, 255, 255,   0,    /* 252 */
    200, 255, 255,   0,    /* 253 */
    229, 255, 255,   0,    /* 254 */
    255, 255, 255,   0    /* 255 - Exact match with default 19 */};

//this function wraps the opengl glgeterror function
void
CheckError()
{
	int err = glGetError();
	if (err == GL_NO_ERROR)
		return;

	switch (err)
	{
	case GL_INVALID_ENUM:
		assert(0); //An unacceptable value is specified for an enumerated argument. The offending function is ignored, having no side effect other than to set the error flag. 
	case GL_INVALID_VALUE:
		assert(0); //A numeric argument is out of range. The offending function is ignored, having no side effect other than to set the error flag. 
	case GL_INVALID_OPERATION:
		assert(0); //The specified operation is not allowed in the current state. The offending function is ignored, having no side effect other than to set the error flag. 
	case GL_STACK_OVERFLOW:
		assert(0); //This function would cause a stack overflow. The offending function is ignored, having no side effect other than to set the error flag. 
	case GL_STACK_UNDERFLOW:
		assert(0); //This function would cause a stack underflow. The offending function is ignored, having no side effect other than to set the error flag. 
	case GL_OUT_OF_MEMORY:
		assert(0); //There is not enough memory left to execute the function. The state of OpenGL is undefined, except for the state of the error flags, after this error is recorded. 
	}
}

//clear any opengl errors
void ClearErrors()
{
	while (glGetError() != GL_NO_ERROR);
}

//initialize opengl to begin rendering to hwnd
void 
GLInit(HWND hWnd)
{
	Window = hWnd;
	RC = NULL;
	DC = GetDC(hWnd);

	PIXELFORMATDESCRIPTOR pfd;
	pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion = 1;
	pfd.dwFlags =	PFD_DRAW_TO_WINDOW | 
					PFD_SUPPORT_OPENGL | 
					PFD_DOUBLEBUFFER |
					PFD_SWAP_COPY;
	
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.cColorBits = 24;
	pfd.cAlphaBits = 16;
	pfd.cDepthBits = 32;
	pfd.cStencilBits = 0;
	pfd.cAuxBuffers = 0;
	pfd.cAccumBits = 0;
	pfd.iLayerType = PFD_MAIN_PLANE;

	int pixelFormat = ChoosePixelFormat(DC, &pfd);		
	ASSERT(pixelFormat != 0);


	int ret = DescribePixelFormat(DC, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
	ASSERT(ret);
	BOOL bRet = SetPixelFormat(DC, pixelFormat, &pfd);
	ASSERT(bRet);

	if ((pfd.dwFlags & 0x80) == 0)
		RGBPalette = 0;
	else
		RGBPalette = CreatePalette((LOGPALETTE *)&rgb8Palette);

	if (RGBPalette != 0)
	{
		SelectPalette(DC, RGBPalette, TRUE);
		RealizePalette(DC);
	}

	RC = wglCreateContext(DC);
	ASSERT(RC);
	bRet = wglMakeCurrent(DC, RC);
	ASSERT(bRet);


	//some opengl flags
	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);

	glAlphaFunc(GL_GREATER, 0.0);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDepthFunc(GL_LEQUAL);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	glPolygonOffset(1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
}

void GLUnInit()
{
	wglDeleteContext(RC);
	ReleaseDC(Window, DC);
}

void SetViewPort(int cx, int cy)
{
	//set the size of the area we're rendering to.
	glViewport(0, 0, cx, cy);
}

void SetLighting(CXGLWorld *world)
{
	//write out our world lighting to the display list
	//the modelview matrix must be identity for the lighting to stay constant as we rotate the world
	glPushMatrix();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glEnable(GL_LIGHTING);
	
	//disable any previous lights
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glDisable(GL_LIGHT2);
	glDisable(GL_LIGHT3);
	glDisable(GL_LIGHT4);
	glDisable(GL_LIGHT5);
	glDisable(GL_LIGHT6);
	glDisable(GL_LIGHT7);

	if (Import.Lighting != NULL)
	{
		GLfloat ambient[] = {Import.Lighting->AmbientR, Import.Lighting->AmbientG, Import.Lighting->AmbientB, 1.0};
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
	
		int i;
		for (i = 0; i < Import.Lighting->NumLights; i++)
		{
			CXGLLight *l = &(Import.Lighting->Lights[i]);
			GLfloat diffuse[] = {l->DiffuseR, l->DiffuseG, l->DiffuseB, 1.0};
			GLfloat specular[] = {l->SpecularR, l->SpecularG, l->SpecularB, 1.0};
			GLfloat position[] = {l->DX, l->DY, l->DZ, 0.0};
		
			switch (i)
			{
			case 0:
				glEnable(GL_LIGHT0);
				glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT0, GL_POSITION, position);
				break;
			case 1:
				glEnable(GL_LIGHT1);
				glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT1, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT1, GL_POSITION, position);
				break;
			case 2:
				glEnable(GL_LIGHT2);
				glLightfv(GL_LIGHT2, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT2, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT2, GL_POSITION, position);
				break;
			case 3:
				glEnable(GL_LIGHT3);
				glLightfv(GL_LIGHT3, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT3, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT3, GL_POSITION, position);
				break;
			case 4:
				glEnable(GL_LIGHT4);
				glLightfv(GL_LIGHT4, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT4, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT4, GL_POSITION, position);
				break;
			case 5:
				glEnable(GL_LIGHT5);
				glLightfv(GL_LIGHT5, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT5, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT5, GL_POSITION, position);
				break;
			case 6:
				glEnable(GL_LIGHT6);
				glLightfv(GL_LIGHT6, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT6, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT6, GL_POSITION, position);
				break;
			case 7:
				glEnable(GL_LIGHT7);
				glLightfv(GL_LIGHT7, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT7, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT7, GL_POSITION, position);
				break;
			}

		}
	}
	else
	{
		GLfloat ambient[] = {0.2f, 0.2f, 0.2f, 1.0f};
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
	}

	glPopMatrix();
}

void TurnOnDoubleSidedRendering()
{
	//if meshes are double-sided (ie. not solid) then we cannot cull faces
	//and we need to light both sides
	glDisable(GL_CULL_FACE);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
}

void TurnOffDoubleSidedRendering()
{
	//solid or single-sided meshs can be lit on only one side and backfacing
	//faces can be culled
	glEnable(GL_CULL_FACE);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
}

void TurnOnNormalTextureFlags()
{
	glEnable(GL_TEXTURE_2D);
}

void TurnOnAlphaTextureFlags()
{
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_ALPHA_TEST);
}

void TurnOffAllTextureFlags()
{
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_ALPHA_TEST);
}

void TurnOnAlphaFlags()
{
	glEnable(GL_BLEND);
	glDepthMask(GL_FALSE);
}

void TurnOffAlphaFlags()
{
	glDisable(GL_BLEND);
	glDepthMask(GL_TRUE);
}

void TurnOffAllFlags()
{
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_ALPHA_TEST);
	glDepthMask(GL_TRUE);
}

void DrawXGLMesh(CXGLMesh *mesh, int faceType, bool forceDoubleSided)
{
	//render the mesh to the display list


	//let opengl know that whether the mesh is solid or not
	if (mesh->DoubleSided == 1 || forceDoubleSided)
		TurnOnDoubleSidedRendering();
	else
		TurnOffDoubleSidedRendering();

	//this is a flag that let's us know whether we need to 
	//changed the material from one face to the next
	unsigned int prevMaterialIndex = -1;
	unsigned int prevTextureIndex = -1;

	TurnOffAllFlags();

	if (faceType == AlphaTextureFaces || faceType == AlphaFaces)
		TurnOnAlphaFlags();

	glEnable(GL_NORMALIZE);

	//begin outputting triangles
	glBegin(GL_TRIANGLES);
	unsigned int i;
	for (i = 0; i < mesh->NumFaces; i++)
	{
		CXGLFace *f = mesh->Faces + i;
		CXGLMaterial *m = mesh->Materials + f->MaterialI;

		//continue on if this face isn't our faceType
		if (f->TextureI != -1)
		{
			if (faceType == NormalFaces || faceType == AlphaFaces)
				continue;
		}
		else
		{
			if (faceType == TextureFaces || faceType == AlphaTextureFaces)
				continue;
		}

		if (m->Alpha != 1.0)
		{
			if (faceType == NormalFaces || faceType == TextureFaces)
				continue;
		}
		else
		{
			if (faceType == AlphaFaces || faceType == AlphaTextureFaces)
				continue;
		}

		//set the texture (if any)
		if (f->TextureI != -1)
		{
			if (prevTextureIndex != f->TextureI)
			{
				int textureID = mesh->GetTextureCallbackHandle(f->TextureI);
				glEnd();
				glCallList(textureID);
				glBegin(GL_TRIANGLES);
				prevMaterialIndex = -1;
			}

			prevTextureIndex = f->TextureI;
		}
	
		//set the material
		if (f->MaterialI != prevMaterialIndex)
		{
			if (prevMaterialIndex != -1)
			{
				if (memcmp(mesh->Materials + f->MaterialI, mesh->Materials + prevMaterialIndex, sizeof(CXGLMaterial))) 
				{
					GLfloat ambient[] = {m->AmbientR, m->AmbientG, m->AmbientB, m->Alpha};
					GLfloat diffuse[] = {m->DiffuseR, m->DiffuseG, m->DiffuseB, m->Alpha};
					GLfloat specular[] = {m->SpecularR, m->SpecularG, m->SpecularB, m->Alpha};
					GLfloat emission[] = {m->EmissiveR, m->EmissiveG, m->EmissiveB, m->Alpha};
					GLfloat shininess[] = {m->Shininess};
					glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
					glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
					glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
					glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
					glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
				}
			}
			else
			{	
				GLfloat ambient[] = {m->AmbientR, m->AmbientG, m->AmbientB, m->Alpha};
				GLfloat diffuse[] = {m->DiffuseR, m->DiffuseG, m->DiffuseB, m->Alpha};
				GLfloat specular[] = {m->SpecularR, m->SpecularG, m->SpecularB, m->Alpha};
				GLfloat emission[] = {m->EmissiveR, m->EmissiveG, m->EmissiveB, m->Alpha};
				GLfloat shininess[] = {m->Shininess};
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
				glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
				glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
			}

			prevMaterialIndex = f->MaterialI;
		}
		
		

		//output the points their normals (if any), and there texture coordinates (if any).
		if (f->FaceVertices[0].NormalI != -1)
			glNormal3f(mesh->Normals[f->FaceVertices[0].NormalI].X, mesh->Normals[f->FaceVertices[0].NormalI].Y, mesh->Normals[f->FaceVertices[0].NormalI].Z);
		if (f->FaceVertices[0].TextureCoordinateI != -1)
			glTexCoord2d(mesh->TextureCoordinates[f->FaceVertices[0].TextureCoordinateI].U, mesh->TextureCoordinates[f->FaceVertices[0].TextureCoordinateI].V);
		glVertex3f(mesh->Vertices[f->FaceVertices[0].VertexI].X, mesh->Vertices[f->FaceVertices[0].VertexI].Y, mesh->Vertices[f->FaceVertices[0].VertexI].Z);

		if (f->FaceVertices[1].NormalI != -1)
			glNormal3f(mesh->Normals[f->FaceVertices[1].NormalI].X, mesh->Normals[f->FaceVertices[1].NormalI].Y, mesh->Normals[f->FaceVertices[1].NormalI].Z);
		if (f->FaceVertices[1].TextureCoordinateI != -1)
			glTexCoord2d(mesh->TextureCoordinates[f->FaceVertices[1].TextureCoordinateI].U, mesh->TextureCoordinates[f->FaceVertices[1].TextureCoordinateI].V);
		glVertex3f(mesh->Vertices[f->FaceVertices[1].VertexI].X, mesh->Vertices[f->FaceVertices[1].VertexI].Y, mesh->Vertices[f->FaceVertices[1].VertexI].Z);

		if (f->FaceVertices[2].NormalI != -1)
			glNormal3f(mesh->Normals[f->FaceVertices[2].NormalI].X, mesh->Normals[f->FaceVertices[2].NormalI].Y, mesh->Normals[f->FaceVertices[2].NormalI].Z);
		if (f->FaceVertices[2].TextureCoordinateI != -1)
			glTexCoord2d(mesh->TextureCoordinates[f->FaceVertices[2].TextureCoordinateI].U, mesh->TextureCoordinates[f->FaceVertices[2].TextureCoordinateI].V);
		glVertex3f(mesh->Vertices[f->FaceVertices[2].VertexI].X, mesh->Vertices[f->FaceVertices[2].VertexI].Y, mesh->Vertices[f->FaceVertices[2].VertexI].Z);
	}

	//we're done with the faces
	glEnd();

	//begin outputting lines
	int prevColInd = -1;

	for (i = 0; i < mesh->NumLines; i++)
	{
		CXGLLine *l = mesh->Lines + i;
		CXGLMaterial *m = mesh->Materials + l->MaterialI;

		//continue on if this line isn't our faceType
		if (l->TextureI != -1)
		{
			if (faceType == NormalFaces || faceType == AlphaFaces)
				continue;
		}
		else
		{
			if (faceType == TextureFaces || faceType == AlphaTextureFaces)
				continue;
		}

		if (m->Alpha != 1.0)
		{
			if (faceType == NormalFaces || faceType == TextureFaces)
				continue;
		}
		else
		{
			if (faceType == AlphaFaces || faceType == AlphaTextureFaces)
				continue;
		}

		// set width of line
		glLineWidth(mesh->LineStyles[l->LineStyleI].Width);  
		// set pattern factor and pattern itself
		if(mesh->LineStyles[l->LineStyleI].Pattern != 0xffff)
		{
			glEnable(GL_LINE_STIPPLE);
			glLineStipple(mesh->LineStyles[l->LineStyleI].PatternFactor, mesh->LineStyles[l->LineStyleI].Pattern);
		}

		//set the texture (if any)
		if (l->TextureI != -1)
		{
			if (prevTextureIndex != l->TextureI)
			{
				int textureID = mesh->GetTextureCallbackHandle(l->TextureI);
				glEnd();
				glCallList(textureID);
				glBegin(GL_LINES);
				prevMaterialIndex = -1;
			}

			prevTextureIndex = l->TextureI;
		}
	
		//set the material or color:
		if(l->LineVertices[0].NormalI == -1)
		{
			//set color instead of material
			glDisable(GL_LIGHTING);
			SetLinePointColor(mesh->Materials, &prevColInd, l->MaterialI);
		}
		else if (l->MaterialI != prevMaterialIndex)
		{
			if (prevMaterialIndex != -1)
			{
				if (memcmp(mesh->Materials + l->MaterialI, mesh->Materials + prevMaterialIndex, sizeof(CXGLMaterial))) 
				{
					GLfloat ambient[] = {m->AmbientR, m->AmbientG, m->AmbientB, m->Alpha};
					GLfloat diffuse[] = {m->DiffuseR, m->DiffuseG, m->DiffuseB, m->Alpha};
					GLfloat specular[] = {m->SpecularR, m->SpecularG, m->SpecularB, m->Alpha};
					GLfloat emission[] = {m->EmissiveR, m->EmissiveG, m->EmissiveB, m->Alpha};
					GLfloat shininess[] = {m->Shininess};
					glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
					glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
					glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
					glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
					glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
				}
			}
			else
			{	
				GLfloat ambient[] = {m->AmbientR, m->AmbientG, m->AmbientB, m->Alpha};
				GLfloat diffuse[] = {m->DiffuseR, m->DiffuseG, m->DiffuseB, m->Alpha};
				GLfloat specular[] = {m->SpecularR, m->SpecularG, m->SpecularB, m->Alpha};
				GLfloat emission[] = {m->EmissiveR, m->EmissiveG, m->EmissiveB, m->Alpha};
				GLfloat shininess[] = {m->Shininess};
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
				glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
				glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
			}

			prevMaterialIndex = l->MaterialI;
		}
	
		glBegin(GL_LINES);

		//output the points their normals (if any), and there texture coordinates (if any).
		if (l->LineVertices[0].NormalI != -1)
			glNormal3f(mesh->Normals[l->LineVertices[0].NormalI].X, mesh->Normals[l->LineVertices[0].NormalI].Y, mesh->Normals[l->LineVertices[0].NormalI].Z);
		if (l->LineVertices[0].TextureCoordinateI != -1)
			glTexCoord2d(mesh->TextureCoordinates[l->LineVertices[0].TextureCoordinateI].U, mesh->TextureCoordinates[l->LineVertices[0].TextureCoordinateI].V);
		glVertex3f(mesh->Vertices[l->LineVertices[0].VertexI].X, mesh->Vertices[l->LineVertices[0].VertexI].Y, mesh->Vertices[l->LineVertices[0].VertexI].Z);
		
		if (l->LineVertices[1].NormalI != -1)
			glNormal3f(mesh->Normals[l->LineVertices[1].NormalI].X, mesh->Normals[l->LineVertices[1].NormalI].Y, mesh->Normals[l->LineVertices[1].NormalI].Z);
		if (l->LineVertices[1].TextureCoordinateI != -1)
			glTexCoord2d(mesh->TextureCoordinates[l->LineVertices[1].TextureCoordinateI].U, mesh->TextureCoordinates[l->LineVertices[1].TextureCoordinateI].V);
		glVertex3f(mesh->Vertices[l->LineVertices[1].VertexI].X, mesh->Vertices[l->LineVertices[1].VertexI].Y, mesh->Vertices[l->LineVertices[1].VertexI].Z);
		
		glEnd();
	
		if(mesh->LineStyles[l->LineStyleI].Pattern != 0xffff)
			glDisable(GL_LINE_STIPPLE);
		
		if(l->LineVertices[0].NormalI == -1)
			glEnable(GL_LIGHTING);
	}

	//we're done with the lines
	//begin outputting points

	for (i = 0; i < mesh->NumPoints; i++)
	{
		CXGLPoint *p = mesh->Points + i;
		CXGLMaterial *m = mesh->Materials + p->MaterialI;
		
		glPointSize(mesh->PointStyles[p->PointStyleI].Size);

		//continue on if this line isn't our faceType
		if (p->TextureI != -1)
		{
			if (faceType == NormalFaces || faceType == AlphaFaces)
				continue;
		}
		else
		{
			if (faceType == TextureFaces || faceType == AlphaTextureFaces)
				continue;
		}

		if (m->Alpha != 1.0)
		{
			if (faceType == NormalFaces || faceType == TextureFaces)
				continue;
		}
		else
		{
			if (faceType == AlphaFaces || faceType == AlphaTextureFaces)
				continue;
		}

		//set the texture (if any)
		if (p->TextureI != -1)
		{
			if (prevTextureIndex != p->TextureI)
			{
				int textureID = mesh->GetTextureCallbackHandle(p->TextureI);
				glEnd();
				glCallList(textureID);
				glBegin(GL_POINTS);
				prevMaterialIndex = -1;
			}

			prevTextureIndex = p->TextureI;
		}
	
		//set the material or color
		if(p->PointVertex.NormalI == -1)
		{
			//set color instead of material
			glDisable(GL_LIGHTING);
			SetLinePointColor(mesh->Materials, &prevColInd, p->MaterialI);
		}
		else if (p->MaterialI != prevMaterialIndex)
		{
			if (prevMaterialIndex != -1)
			{
				if (memcmp(mesh->Materials + p->MaterialI, mesh->Materials + prevMaterialIndex, sizeof(CXGLMaterial))) 
				{
					GLfloat ambient[] = {m->AmbientR, m->AmbientG, m->AmbientB, m->Alpha};
					GLfloat diffuse[] = {m->DiffuseR, m->DiffuseG, m->DiffuseB, m->Alpha};
					GLfloat specular[] = {m->SpecularR, m->SpecularG, m->SpecularB, m->Alpha};
					GLfloat emission[] = {m->EmissiveR, m->EmissiveG, m->EmissiveB, m->Alpha};
					GLfloat shininess[] = {m->Shininess};
					glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
					glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
					glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
					glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
					glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
				}
			}
			else
			{	
				GLfloat ambient[] = {m->AmbientR, m->AmbientG, m->AmbientB, m->Alpha};
				GLfloat diffuse[] = {m->DiffuseR, m->DiffuseG, m->DiffuseB, m->Alpha};
				GLfloat specular[] = {m->SpecularR, m->SpecularG, m->SpecularB, m->Alpha};
				GLfloat emission[] = {m->EmissiveR, m->EmissiveG, m->EmissiveB, m->Alpha};
				GLfloat shininess[] = {m->Shininess};
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
				glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
				glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
			}

			prevMaterialIndex = p->MaterialI;
		}

		glBegin(GL_POINTS);

		//output the points their normals (if any), and there texture coordinates (if any).
		if (p->PointVertex.NormalI != -1)
			glNormal3f(mesh->Normals[p->PointVertex.NormalI].X, mesh->Normals[p->PointVertex.NormalI].Y, mesh->Normals[p->PointVertex.NormalI].Z);
		if (p->PointVertex.TextureCoordinateI != -1)
			glTexCoord2d(mesh->TextureCoordinates[p->PointVertex.TextureCoordinateI].U, mesh->TextureCoordinates[p->PointVertex.TextureCoordinateI].V);
		glVertex3f(mesh->Vertices[p->PointVertex.VertexI].X, mesh->Vertices[p->PointVertex.VertexI].Y, mesh->Vertices[p->PointVertex.VertexI].Z);
	
		glEnd();

		if(p->PointVertex.NormalI == -1)
			glEnable(GL_LIGHTING);
	}

	glDisable(GL_NORMALIZE);

	TurnOffAllFlags();
}

void
SetLinePointColor(CXGLMaterial *matArray, int *prevInd, int newInd)
{
	GLfloat r, g, b, a;

	CXGLMaterial *mat = matArray + newInd;

	if(*prevInd == -1 || 
	   (*prevInd != newInd && memcmp(mat, matArray + *prevInd, sizeof(CXGLMaterial)) != 0))
	{
		// calculate color:
		r = __max(mat->AmbientR, __max(mat->DiffuseR, mat->EmissiveR));
		g = __max(mat->AmbientG, __max(mat->DiffuseG, mat->EmissiveG));
		b = __max(mat->AmbientB, __max(mat->DiffuseB, mat->EmissiveB));

		GLfloat maxCol = __max(r, __max(g, b));

		if(maxCol < 0.9375)
		{
			r += mat->SpecularR * 0.0625f;
			g += mat->SpecularG * 0.0625f;
			b += mat->SpecularB * 0.0625f;
		}

		a = mat->Alpha;

		glColor4f(r, g, b, a);
	}

	*prevInd = newInd;
}

void DrawXGLObject(CXGLObject *object, int faceType)
{
	//create the object to world transform in opengl for the object about to be rendered
	CXGLTransform *t = &(object->Transform);
	double ux = t->UX;
	double uy = t->UY;
	double uz = t->UZ;
	double fx = t->FX;
	double fy = t->FY;
	double fz = t->FZ;
	double s = t->Scale;
	double xp = t->XP;
	double yp = t->YP;
	double zp = t->ZP;
	
	float xDZ = (float)(fx * s);
	float yDZ = (float)(fy * s);
	float zDZ = (float)(fz * s);

	double fDotu = fx*ux + fy*uy + fz*uz;
	ASSERT(fDotu != 1.0);
	double yx = ux - fx * fDotu;
	double yy = uy - fy * fDotu;
	double yz = uz - fz * fDotu;
	double invYMag = 1 / sqrt(yx*yx + yy*yy + yz*yz);
	yx = yx * invYMag;
	yy = yy * invYMag;
	yz = yz * invYMag;

	float xDY = (float)(yx * s);
	float yDY = (float)(yy * s);
	float zDY = (float)(yz * s);

	float xDX = (float)((yy*fz - yz*fy) * s);
	float yDX = (float)((yz*fx - yx*fz) * s);
	float zDX = (float)((yx*fy - yy*fx) * s);

	float xPos = (float)xp;
	float yPos = (float)yp;
	float zPos = (float)zp;

	//transform the object from objectspace to worldspace	
	GLfloat m[16] = {
		  xDX,
		  yDX,
		  zDX,
		  0,
		  xDY,
		  yDY,
		  zDY,
		  0,
		  xDZ,
		  yDZ,
		  zDZ,
		  0,
		  xPos,
		  yPos,
		  zPos,
		  1
	};

	//set the transform
	glMultMatrixf(m);
	
	//render the object's mesh
	int meshIndex = object->GetMeshCallbackHandle();
	if (meshIndex >= 0)
	{
		CDisplayListNode *mesh = Import.DisplayListNodes.GetElementRef(meshIndex);
		if (faceType == NormalFaces && mesh->NormalFaces)
			glCallList(mesh->NormalFaces);
		if (faceType == TextureFaces && mesh->TextureFaces)
			glCallList(mesh->TextureFaces);
		if (faceType == AlphaTextureFaces && mesh->AlphaTextureFaces)
			glCallList(mesh->AlphaTextureFaces);
		if (faceType == AlphaFaces && mesh->AlphaFaces)
			glCallList(mesh->AlphaFaces);
	}

	//draw any children of this object
	unsigned int i;
	for (i = 0; i < object->NumObjects; i++)
	{
		int objectIndex = object->GetObjectCallbackHandle(i);
		if (objectIndex >= 0)
		{
			CDisplayListNode *child = Import.DisplayListNodes.GetElementRef(objectIndex);
			if (faceType == NormalFaces && child->NormalFaces)
			{
				glPushMatrix();
				glCallList(child->NormalFaces);
				glPopMatrix();
			}

			if (faceType == TextureFaces && child->TextureFaces)
			{
				glPushMatrix();
				glCallList(child->TextureFaces);
				glPopMatrix();
			}

			if (faceType == AlphaTextureFaces && child->AlphaTextureFaces)
			{
				glPushMatrix();
				glCallList(child->AlphaTextureFaces);
				glPopMatrix();
			}

			if (faceType == AlphaFaces && child->AlphaFaces)
			{
				glPushMatrix();
				glCallList(child->AlphaFaces);
				glPopMatrix();
			}
		}
	}
}

void DrawXGLWorld(CXGLWorld *world)
{
	//set the lighting in the display list
	SetLighting(world);

	//render the world's mesh (if any)
	int meshIndex = world->GetMeshCallbackHandle();
	if (meshIndex >= 0)
	{
		assert(world->NumObjects == 0);

		CDisplayListNode *mesh = Import.DisplayListNodes.GetElementRef(meshIndex);
		if (mesh->NormalFaces)
			glCallList(mesh->NormalFaces);
		if (mesh->TextureFaces)
			glCallList(mesh->TextureFaces);
		if (mesh->AlphaTextureFaces)
			glCallList(mesh->AlphaTextureFaces);
		if (mesh->AlphaFaces)
			glCallList(mesh->AlphaFaces);
	}
	else
	{
		unsigned int i;
		for (i = 0; i < world->NumObjects; i++)
		{
			int objectIndex = world->GetObjectCallbackHandle(i);
			if (objectIndex >= 0)
			{
				CDisplayListNode *object = Import.DisplayListNodes.GetElementRef(objectIndex);
				
				if (object->NormalFaces)
				{
					glPushMatrix();
					glCallList(object->NormalFaces);
					glPopMatrix();
				}
			}
		}

		for (i = 0; i < world->NumObjects; i++)
		{
			int objectIndex = world->GetObjectCallbackHandle(i);
			if (objectIndex >= 0)
			{
				CDisplayListNode *object = Import.DisplayListNodes.GetElementRef(objectIndex);

				if (object->TextureFaces)
				{
					glPushMatrix();
					glCallList(object->TextureFaces);
					glPopMatrix();
				}
			}
		}

		for (i = 0; i < world->NumObjects; i++)
		{
			int objectIndex = world->GetObjectCallbackHandle(i);
			if (objectIndex >= 0)
			{
				CDisplayListNode *object = Import.DisplayListNodes.GetElementRef(objectIndex);

				if (object->AlphaTextureFaces)
				{
					glPushMatrix();
					glCallList(object->AlphaTextureFaces);
					glPopMatrix();
				}
			}
		}

		for (i = 0; i < world->NumObjects; i++)
		{
			int objectIndex = world->GetObjectCallbackHandle(i);
			if (objectIndex >= 0)
			{
				CDisplayListNode *object = Import.DisplayListNodes.GetElementRef(objectIndex);

				if (object->AlphaFaces)
				{
					glPushMatrix();
					glCallList(object->AlphaFaces);
					glPopMatrix();
				}
			}
		}
	}
}

unsigned int 
CreateMeshDisplayList(CXGLMesh *mesh, int faceType, bool forceDoubleSided)
{
	assert(mesh != NULL);

	//make sure all of the commands in our previous list are completed
	glFinish();

	//create the new list;
	unsigned int ret = glGenLists(1);
	glNewList(ret, GL_COMPILE);
	if (ret >= DisplayListRange)
		DisplayListRange = ret + 1;

	//draw the mesh to the list
	DrawXGLMesh(mesh, faceType, forceDoubleSided);

	//finish the list
	glEndList();

	//wait for opengl to finish
	glFinish();

	return ret;
}

unsigned int
CreateTextureBuffer(CXGLTextureBuffer *textureBuffer)
{
	//wait for opengl 
	glFinish();

	GLuint ret = 0;
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &ret);

	glBindTexture(GL_TEXTURE_2D, ret);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 
			     textureBuffer->TextureWidth,
				 textureBuffer->TextureHeight,
				 0, GL_RGBA, GL_UNSIGNED_BYTE, textureBuffer->RGBABuffer);
	
	//wait for opengl
	glFinish();

	return ret;
}

unsigned int
CreateTexture(CXGLTexture *texture)
{
	int textureBufferID = texture->GetTextureBufferCallbackHandle();
	assert(textureBufferID != -1);

	glBindTexture(GL_TEXTURE_2D, textureBufferID);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	switch(texture->TextureWrap)
	{
	case TextureRepeat:
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		break;
	case TextureClamp:
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		break;
	}
	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	if (texture->UseTextureBorderColor == 1)
		glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, texture->TextureBorderColor);

	switch (texture->TextureFunction)
	{
	case TextureReplace:
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		break;
	case TextureModulate:
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		break;
	case TextureDecal:
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
		break;
	}


	//make sure all of the commands in our previous list are completed
	glFinish();

	//create the new list;
	unsigned int ret = glGenLists(1);
	glNewList(ret, GL_COMPILE);
	if (ret >= DisplayListRange)
		DisplayListRange = ret + 1;

	//bind the texture and turn on the flags
	
	switch(texture->TextureBufferFormat)
	{
	case TextureRGB:
		TurnOnNormalTextureFlags();
		break;
	case TextureRGBA:
		TurnOnAlphaTextureFlags();
		break;
	}

	glBindTexture(GL_TEXTURE_2D, textureBufferID);

	//finish the list
	glEndList();

	//wait for opengl to finish
	glFinish();

	return ret;
}

unsigned int 
CreateObjectDisplayList(CXGLObject *object, int faceType)
{
	assert(object != NULL);

	//make sure all of the commands in our previous list are completed
	glFinish();

	//create the new list;
	unsigned int ret = glGenLists(1);
	glNewList(ret, GL_COMPILE);
	if (ret >= DisplayListRange)
		DisplayListRange = ret + 1;

	//draw the object to the list
	DrawXGLObject(object, faceType);

	//finish the list
	glEndList();

	//wait for opengl to finish
	glFinish();

	return ret;
}

void CreateWorldDisplayList(CXGLWorld *world)
{
	//make sure all of the commands in our previous list are completed
	glFinish();
	
	//set the background color
	if (Import.Background != NULL)
		glClearColor(Import.Background->Red, Import.Background->Green, Import.Background->Blue, 0.0);
	else
		glClearColor(0.0, 0.0, 0.0, 0.0);

	//clear our buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//create the new list
	WorldID = glGenLists(1);
	if (WorldID >= DisplayListRange)
		DisplayListRange = WorldID + 1;
	glNewList(WorldID, GL_COMPILE);

	//draw the world to the list
	DrawXGLWorld(world);

	//finish the list
	glEndList();

	//wait for opengl to finish
	glFinish();
}

void DeleteDisplayLists()
{
	//set a white background
	glClearColor(1.0, 1.0, 1.0, 0.0);

	//clear our buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//delete all display lists
	glDeleteLists(1, DisplayListRange);
	DisplayListRange = 1;

	//glDeleteTextures(TextureObjectRange, 

	//write the white screen to the window
	SwapBuffers(DC);
}

void CallDisplayList()
{
	//we can only draw if we have a display list
	if (glIsList(WorldID) == GL_TRUE)
	{
		//clear the buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		//set up the rotation matrix of the entire world
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		float rx = UY*FZ - UZ*FY;
		float ry = UZ*FX - UX*FZ;
		float rz = UX*FY - UY*FX;

		//translate the world to the center
		GLfloat translate[16] = {
			1.0,
			0.0,
			0.0,
			0.0,
			0.0,
			1.0,
			0.0,
			0.0,
			0.0,
			0.0,
			1.0,
			0.0,
			XP,
			YP,
			ZP,
			1};
	

		//Rotate the top transform
		GLfloat m[16] = {
					rx,
					ry,
					rz,
					0,
					UX,
					UY,
					UZ,
					0,
					FX,
					FY,
					FZ,
					0,
					0,
					0,
					0,
					1};

		//rotates the world
		glMultMatrixf(m);
		//translates the world
		glMultMatrixf(translate);

		//call the display list
		glCallList(WorldID);

		//wait for opengl to finish
		glFinish();

		//write to the window
		SwapBuffers(DC);
	}
}

void SetGLOrtho(double left, double right, double bottom, double top, double n, double f)
{
	//set up our view frustum
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//compensate for the dimensions of our window
	RECT r;
	GetClientRect(Window, &r);
	double d1 = r.bottom - r.top;
	double d2 = r.right - r.left;

	double screenAspect = d2 / d1;
	double viewAspect = (right - left) / (top - bottom);

	if (screenAspect > viewAspect)
	{
		double hcenter = (right + left) * 0.5;
		double hwidth = (right - left) * 0.5;
		right = hcenter + hwidth * screenAspect;
		left = hcenter - hwidth * screenAspect;

		double vcenter = (bottom + top) * 0.5;
		bottom = vcenter - hwidth;
		top = vcenter + hwidth;
	}
	else
	{
		double vcenter = (bottom + top) * 0.5;
		double vwidth = (top - bottom) * 0.5;
		bottom = vcenter - vwidth / screenAspect;
		top = vcenter + vwidth / screenAspect;

		double hcenter = (right + left) * 0.5;
		right = hcenter + vwidth;
		left = hcenter - vwidth;
	}

	glOrtho(left, right, bottom, top, n, f);

	//redraw the list
	CallDisplayList();
}

void RotateWorld(double fx, double fy, double fz, double ux, double uy, double uz)
{
	//rotate the world
	FX = (float)fx;
	FY = (float)fy;
	FZ = (float)fz;
	UX = (float)ux;
	UY = (float)uy;
	UZ = (float)uz;

	//redraw the list
	CallDisplayList();
}

void TranslateWorld(double xp, double yp, double zp)
{
	//translate the world
	XP = (float)xp;
	YP = (float)yp;
	ZP = (float)zp;

	//redraw the list
	CallDisplayList();
}
