#include "stdafx.h"
#include "ScaleImage.h"
#include <assert.h>
#include <math.h>

class CPixel
{
public:
	unsigned char R;
	unsigned char G;
	unsigned char B;
	unsigned char A;
};

class CPixelAccumulator
{
public:
	double R;
	double G;
	double B;
	double A;
	double TotalAmount;
};

void
ClearAccumulator(CPixelAccumulator *accum)
{
	accum->R = 0;
	accum->G = 0;
	accum->B = 0;
	accum->A = 0;
	accum->TotalAmount = 0;
}

void
SetPixelToAccululator(CPixel *p, CPixelAccumulator *accum)
{
	if (accum->TotalAmount > 0)
	{
		//p->A = (unsigned char) __min(floor(accum->A / 255 + 0.5) * 255, 255);
		p->A = (unsigned char) __min(accum->A, 255);
/*
		if (p->A == 0)
		{
			p->R = 0;
			p->G = 0;
			p->B = 0;
		}
		else
*/
		{
			p->R = (unsigned char) __min(accum->R / accum->TotalAmount, 255);
			p->G = (unsigned char) __min(accum->G / accum->TotalAmount, 255);
			p->B = (unsigned char) __min(accum->B / accum->TotalAmount, 255);
		}
	}
	else
	{
		p->R = (unsigned char) 0;
		p->G = (unsigned char) 0;
		p->B = (unsigned char) 0;
		p->A = (unsigned char) 0;
	}
}


void
AccumulatePixel(CPixelAccumulator *accum, const CPixel *oldP, double amount)
{
	accum->R += (oldP->A / 255.0) * oldP->R * amount;
	accum->G += (oldP->A / 255.0) * oldP->G * amount;
	accum->B += (oldP->A / 255.0) * oldP->B * amount;
	accum->A += oldP->A * amount;


	// Keep track of how much has been applied so that the color can be scaled up
	//  appropriately.

	accum->TotalAmount += amount * (oldP->A / 255.0);
}

void
ScaleLine(int numPixels, const CPixel *line, int pixelStride, int newNumPixels, CPixel *newLine, int newPixelStride)
{
	double ratio = (double) newNumPixels / (double) numPixels;
	double amountLeftToRead = ratio;
	double amountLeftToApply = 1;
	const CPixel *oldP = line;
	CPixel *newP = newLine;

	CPixelAccumulator accum;
	ClearAccumulator(&accum);

	CPixel *endP = newLine + newPixelStride * newNumPixels;
	while (newP < endP)
	{
		if (amountLeftToApply < amountLeftToRead)
		{

			// Apply all and advance new

			AccumulatePixel(&accum, oldP, amountLeftToApply);
			SetPixelToAccululator(newP, &accum);
			ClearAccumulator(&accum);
			
			
			amountLeftToRead -= amountLeftToApply;
			amountLeftToApply = 1;
			newP += pixelStride;

			assert(newP <= endP);
		}
		else
		{

			// Apply Remaining and advance old

			AccumulatePixel(&accum, oldP, amountLeftToRead);

			amountLeftToApply -= amountLeftToRead;
			amountLeftToRead = ratio;
			
			if (oldP < line + pixelStride * (numPixels - 1))
				oldP += pixelStride;
		}
	}

	assert(newP == endP);
}

void
FloorAlpha(CPixel *p)
{
	if (p->A < 128)
	{
		p->R = 0;
		p->G = 0;
		p->B = 0;
		p->A = 0;
	}
	else
	{
		p->A = 255;
	}
}


void
ScaleImage(int width, int height, const unsigned char *rgbImage, int newWidth, int newHeight, unsigned char *newRGBImage)
{

	// Set the pixel images

	CPixel *image = (CPixel *) rgbImage;
	CPixel *newImage = (CPixel *) newRGBImage;
	CPixel *tempImage = new CPixel[newWidth*height];

	
	// Scale Horizontally

	int y;
	for (y = 0; y < height; y ++)
	{
		ScaleLine(width, image + y*width, 1, newWidth, tempImage + y*newWidth, 1);
	}


	// Scale Vertically

	int x;
	for (x = 0; x < newWidth; x ++)
	{
		ScaleLine(height, tempImage + x, newWidth, newHeight, newImage + x, newWidth);
	}

	delete[] tempImage;


	for (y = 0; y < newHeight; y ++)
	{
		for (x = 0; x < newWidth; x ++)
		{
			FloorAlpha(newImage+x+y*newWidth);
		}
	}

}
